#############################################################################
#
# MD on November 20 2006
# TCL script to generate the SLS update XML files for DIP DNS 1, 2 and Meta -level.
#
#############################################################################
# The steps :
#
# Read properties.
# Compute availabilities.
# Dump XML files.
#	
#############################################################################

#puts "\nStarting the SLS XML writer..."
#puts "================================\n"

#File containing the CVS log output
set inputFilePrimary    "nsCountPrimary.txt"
set inputFileSecondary  "nsCountSecondary.txt"
set inputFileProperties "XMLUpdateProperties.txt"

#Availabilities.
set availability1    ""
set availability2    ""
set availabilityMeta ""

#DNS characteristics
set DNSName1 ""
set DateAndTime1 ""
set NoPublications1 -1
set ExceptionText1 "Fine"
set DNSName2 ""
set DateAndTime2 ""
set NoPublications2 -1
set ExceptionText2 "Fine"
set excludedXMLUpdates ""
set ouputUpdatedXMLFile1 	"DIPDNS1Update.xml"
set ouputUpdatedXMLFile2 	"DIPDNS2Update.xml"
set ouputUpdatedXMLFileMeta 	"DIPDNSMETAUpdate.xml"
set availabilityInfo1 ""
set availabilityInfo2 ""
set availabilityInfoMeta ""
set metaPublications ""

#Local Data
set convertedCurrentDateAndTime ""
set currentXSDateTime ""
set DNS1NotAvailable 0
set DNS2NotAvailable 0

#To be read from properties file
set interventionsStartDNS1	""
set interventionsLengthDNS1	""
set interventionsDescDNS1	""
set incidentDNS1 		""
set interventionsStartDNS2	""
set interventionsLengthDNS2	""
set interventionsDescDNS2	""
set incidentDNS2 		""
set metaInterventionsStart	""
set metaInterventionsLength	""
set metaInterventionsDesc	""
set incidentMeta 		""
set SMSList	""
set EmergencySMSList	""


#############################################################################
# proc openFile {arg1}
#############################################################################
# Just returns a handle on a file.
proc openFile {arg1} {
	set infile [open $arg1 r]
	return $infile
}

#############################################################################
# proc readInputPropertyFile {}
#############################################################################
# Read the dynamic properties for update XML files generation.
proc readInputPropertyFile {} {
	variable inputFileProperties
	#From file: 
	variable interventionsStartDNS1
	variable interventionsLengthDNS1
	variable interventionsDescDNS1
	variable incidentDNS1
	
	variable interventionsStartDNS2
	variable interventionsLengthDNS2
	variable interventionsDescDNS2
	variable incidentDNS2
	
	variable metaInterventionsStart
	variable metaInterventionsLength
	variable metaInterventionsDesc
	variable incidentMeta

	variable SMSList
	variable EmergencySMSList


	set theFile [openFile $inputFileProperties]
	while { [gets $theFile line] >= 0 } {
		if { [string first "#" [lindex $line 0] ] >= 0 || [string match "" $line] } {
			#skip the commented lines.
			continue
		} else {
			#Process the line
			#Check if the name of the property is known
			if { [string match "interventionsStartDNS1" [lindex $line 0] ] } {
				set interventionsStartDNS1 [lrange $line 1 end]
				#puts "interventionsStartDNS1  = $interventionsStartDNS1"
				continue
			} elseif { [string match "interventionsLengthDNS1" [lindex $line 0] ] } {
				set interventionsLengthDNS1 [lrange $line 1 end]
				#puts "interventionsLengthDNS1  = $interventionsLengthDNS1"
				continue
			} elseif { [string match "interventionsDescDNS1" [lindex $line 0] ] } {
				set interventionsDescDNS1 [lrange $line 1 end]
				#puts "interventionsDescDNS1  = $interventionsDescDNS1"
				continue
			} elseif { [string match "incidentDNS1" [lindex $line 0] ] } {
				set incidentDNS1 [lrange $line 1 end]
				#puts "incidentDNS1  = $incidentDNS1"
				continue
			} elseif { [string match "interventionsStartDNS2" [lindex $line 0] ] } {
				set interventionsStartDNS2 [lrange $line 1 end]
				#puts "interventionsStartDNS2  = $interventionsStartDNS2"
				continue
			} elseif { [string match "interventionsLengthDNS2" [lindex $line 0] ] } {
				set interventionsLengthDNS2 [lrange $line 1 end]
				#puts "interventionsLengthDNS2  = $interventionsLengthDNS2"
				continue
			} elseif { [string match "interventionsDescDNS2" [lindex $line 0] ] } {
				set interventionsDescDNS2 [lrange $line 1 end]
				#puts "interventionsDescDNS2  = $interventionsDescDNS2"
				continue
			} elseif { [string match "incidentDNS2" [lindex $line 0] ] } {
				set incidentDNS2 [lrange $line 1 end]
				#puts "incidentDNS2  = $incidentDNS2"
				continue
			} elseif { [string match "metaInterventionsStart" [lindex $line 0] ] } {
				set metaInterventionsStart [lrange $line 1 end]
				#puts "metaInterventionsStart  = $metaInterventionsStart"
				continue
			} elseif { [string match "metaInterventionsLength" [lindex $line 0] ] } {
				set metaInterventionsLength [lrange $line 1 end]
				#puts "metaInterventionsLength  = $metaInterventionsLength"
				continue
			} elseif { [string match "metaInterventionsDesc" [lindex $line 0] ] } {
				set metaInterventionsDesc [lrange $line 1 end]
				#puts "metaInterventionsDesc  = $metaInterventionsDesc"
				continue
			} elseif { [string match "incidentMeta" [lindex $line 0] ] } {
				set incidentMeta [lrange $line 1 end]
				#puts "incidentMeta  = $incidentMeta"
				continue
			} elseif { [string match "mailingList" [lindex $line 0] ] } {
				set SMSList [lrange $line 1 end]
				continue
			} elseif { [string match "emergencyMailingList" [lindex $line 0] ] } {
				set EmergencySMSList [lrange $line 1 end]
				continue
			} else {
				#Do nothing
				#puts "WARNING: Unknown parameters passed as properties."
			}
		}
	}
}

#############################################################################
# proc readInputDataFiles {}
#############################################################################
# Read the input files and initialize the related lists.
proc readInputDataFiles {} {

	variable inputFilePrimary
	variable inputFileSecondary
	variable DNSName1
	variable DateAndTime1
	variable NoPublications1
	variable Exception1
	variable DNSName2
	variable DateAndTime2
	variable NoPublications2
	variable Exception2

	#Reading the input file connection result.
	
	set theFile1 [openFile $inputFilePrimary]
	while { [gets $theFile1 line] >= 0 } {
		set tagTemplate ""
		set tagLevel ""
		if { [string first "#" [lindex $line 0] ] >= 0 || [string match "" $line] } {
			#skip the commented lines.
			continue
		} else {
			#Process the line
			#Check if the name of the property is known
			if { [string match "DNSName" [lindex $line 0] ] } {
				set DNSName1 [lindex $line 1]
				#puts "DNSName1  = $DNSName1"
				continue
			} elseif { [string match "DateAndTime" [lindex $line 0] ] } {
				#set DateAndTime1 [lindex $line 1]
				set DateAndTime1 [lrange $line 1 end]
				#puts "DateAndTime1 = $DateAndTime1"
				continue
			} elseif { [string match "NoPublications" [lindex $line 0] ] } {
				set NoPublications1 [lindex $line 1]
				#puts "NoPublications1 = $NoPublications1"
				continue
			} elseif { [string match "ExceptionText" [lindex $line 0] ] } {
				set ExceptionText1 [lindex $line 1]
				#puts "ExceptionText1 = $ExceptionText1"
				continue
			} else {
				#Do nothing
				set value [lindex $line 0]
				#puts "Unrecognized tag for Primary: $value"
			}
		}
	}
	close $theFile1
	set theFile2 [openFile $inputFileSecondary]
	while { [gets $theFile2 line] >= 0 } {
		set tagTemplate ""
		set tagLevel ""
		if { [string first "#" [lindex $line 0] ] >= 0 || [string match "" $line] } {
			#skip the commented lines.
			continue
		} else {
			#Process the line
			#Check if the name of the property is known
			if { [string match "DNSName" [lindex $line 0] ] } {
				set DNSName2 [lindex $line 1]
				#puts "DNSName2  = $DNSName2"
				continue
			} elseif { [string match "DateAndTime" [lindex $line 0] ] } {
				set DateAndTime2 [lrange $line 1 end]
				#puts "DateAndTime2 = $DateAndTime2"
				continue
			} elseif { [string match "NoPublications" [lindex $line 0] ] } {
				set NoPublications2 [lindex $line 1]
				#puts "NoPublications2 = $NoPublications2"
				continue
			} elseif { [string match "ExceptionText" [lindex $line 0] ] } {
				set ExceptionText2 [lindex $line 1]
				#puts "ExceptionText2 = $ExceptionText2"
				continue
			} else {
				#Do nothing
				set value [lindex $line 0]
				#puts "Unrecognized tag for Primary: $value"
			}
		}
	}
	close $theFile2
}

#############################################################################
# proc computeAvalability {DNSName DateAndTime Exception NoPublications}
#############################################################################
# Computes the level of availability of a particular DNS (Primary or Secondary).
proc computeAvalability {DNSName DateAndTime Exception NoPublications} {
	variable excludedXMLUpdates
	variable availabilityInfo1
	variable availabilityInfo2
	variable DNSName1
	variable DNSName2

	set availability -1
	set timeValid [ computeTimeDeltaValidity $DateAndTime ]
	set excluded 0
	set availabilityInfo ""	

	if { $timeValid == 1 && [string match $Exception "Fine"] != 1 &&  $NoPublications == 0 } {
		#An exception occured and the publications number could not be determined.
		set availability 0
		set availabilityInfo "An exception occured while contacting the DNS, incorrect response."
	} elseif { $timeValid == 1 && [string match $Exception "Fine"] == 1 &&  $NoPublications == 0 } {
		#No exception occured but the publications number could not be determined.
		set availability 0
		set availabilityInfo "The DNS provided an invalid response or is not accessible."
	} elseif { $timeValid == 1 && [string match $Exception "Fine"] != 1 &&  $NoPublications > 0 } {
		#An exception occured but the publications number could be determined.
		set availability 70
		set availabilityInfo "An exception occured while contacting the DNS, correct response."
	} elseif { $timeValid == 1 && [string match $Exception "Fine"] == 1 &&  $NoPublications > 0 } {
		#Timing OK, no exception, publication number is fine.
		set availability 100
		set availabilityInfo "DNS was contacted successfully and response is valid."
	} else {
		#Strange state, something isn`t correct. (we should`nt get there)
		set excludedXMLUpdates [ lappend excludedXMLUpdates $DNSName ]
		set excluded  1
		set availability 0	
		set availabilityInfo "For external reasons the DNS could not be probed, status is unknown."
	}
	
	if {[string match $DNSName $DNSName1] == 1} {
		#Processed DNS1
		set availabilityInfo1 $availabilityInfo

	} else {
		#Processed DNS2
		set availabilityInfo2 $availabilityInfo
	}

	#puts "For $DNSName, timeValid = $timeValid, DateAndTime = $DateAndTime, Exception = $Exception and NoPublications = $NoPublications so availability = $availability and excluded = $excluded"
	return $availability
}

#############################################################################
# proc computeAvalabilityMeta {availability1 availability2}
#############################################################################
# Computes the Meta -level of availability based on the ones computed for the Primary and the Secondary DNS.
proc computeAvalabilityMeta {availability1 availability2} {
	variable DNSName1
	variable DNSName2
	variable excludedXMLUpdates
	variable DNS1NotAvailable 
	variable DNS2NotAvailable 
	variable availabilityInfoMeta

	set metaAvailability -1
	
	#Check if one of the availability is not accessible. This means a status 1/2 DNS(s) could not be determined.
	foreach i $excludedXMLUpdates {
		if {[string match $i $DNSName1] == 1} {
			set DNS1NotAvailable 1
		} elseif {[string match $i $DNSName2] == 1} {
			set DNS2NotAvailable 1
		}
	}
	
	#puts "excludedXMLUpdates = $excludedXMLUpdates"
	#puts "availability1      = $availability1"
	#puts "availability2      = $availability2"
	
	if {$DNS1NotAvailable == 0 && $DNS1NotAvailable == 0 && $availability1 == 100 && $availability2 == 100} {
		#All fine.
		set availabilityInfoMeta "Both DNS were contacted successfully and responses are valid."
		set metaAvailability 100
		#puts "All fine."
	} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 0 && $availability1 == 100 && $availability2==0 } {
		#Then there is no backup DNS available to the DIP users, not safe.
		set availabilityInfoMeta "Primary DNS is available but backup DNS isn`t."
		set metaAvailability 70
		#puts "Then there is no backup DNS available to the DIP users, not safe."
	} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 0 && $availability1 == 100 && $availability2==70 } {
		#Then the backup DNS thrown an exception but seems available to the DIP users.
		set availabilityInfoMeta "Primary DNS is available, the backup DNS thrown an exception but is available."
		set metaAvailability 98
		#puts "Then the backup DNS thrown an exception but seems available to the DIP users."
	} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 0 && $availability1 == 70 && $availability2==100 } {
		#Then the backup DNS is available to the DIP users but the primary thrown an exception but seems available.
		set availabilityInfoMeta "Primary DNS thrown an exception but is available, the backup DNS is available."
		set metaAvailability 80
		#puts "Then the backup DNS is available to the DIP users but the primary thrown an exception but seems available."
	} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 0 && $availability1 == 0 && $availability2==100 } {
		#Then everybody is running on the backup DNS, not safe.
		set availabilityInfoMeta "Primary DNS isn`t available, the backup DNS is available."
		set metaAvailability 70
		#puts "Then everybody is running on the backup DNS, not safe."
	} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 0 && $availability1 == 0 && $availability2==0 } {
		#Worse case.
		set availabilityInfoMeta "DIP service is not available."
		set metaAvailability 0
		#puts "Worse case."
	} elseif { $DNS1NotAvailable == 1 && $DNS2NotAvailable == 0 } {
		#Could not deduce anything from DNS1, show half of the availability of DNS2. (we should`nt get there)
		set availabilityInfoMeta "Unknown status of primary DNS, backup DNS is present."
		set metaAvailability [expr {$availability2*0.9}]
		#puts "Could not deduce anything from DNS1, show half of the availability of DNS2. (we should`nt get there)"
	} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 1 } {
		#Could not deduce anything from DNS2, show half of the availability of DNS1. (we should`nt get there)
		set availabilityInfoMeta "Unknown status of backup DNS, primary DNS is present."
		set metaAvailability [expr {$availability1*0.9}]
		#puts "Could not deduce anything from DNS2, show half of the availability of DNS1. (we should`nt get there)"
	} elseif { $DNS1NotAvailable == 1 && $DNS2NotAvailable == 1 } {
		#Nothing could be computed, append the metaAvailability to the list of not available.
		set availabilityInfoMeta "Unknown status of both primary and backup DNS."
		set excludedXMLUpdates [ lappend excludedXMLUpdates "Meta" ]
		set metaAvailability 0
		#puts "Nothing could be computed, append the META -Availability to the list of not available."
	} else {
		#We should`nt get there.
		#puts "We should`nt get there."
	}

	return $metaAvailability
}

#############################################################################
# proc computeTimeDeltaValidity {fileDateAndTime}
#############################################################################
# Return 1 if the time provided is < current time by less than 3 minute. (hardcoded, that`s life...)
proc computeTimeDeltaValidity {fileDateAndTime} {
	variable currentXSDateTime
	variable convertedCurrentDateAndTime
	set timeValid 0
	set convertedFileDateAndTime [clock scan $fileDateAndTime]
	set currentDateAndTime [clock format [clock seconds] -format {%H:%M:%S %d %B %y}]
	#Same but format is: YYYY-MM-DDThh:mm:ss
	set currentXSDateTime  [clock format [clock seconds] -format {%Y-%m-%dT%H:%M:%S}]
	set convertedCurrentDateAndTime [clock scan $currentDateAndTime]
	
	#puts "fileDateAndTime             = $fileDateAndTime"
	#puts "currentDateAndTime          = $currentDateAndTime"
	#puts "convertedFileDateAndTime    = $convertedFileDateAndTime"
	#puts "convertedCurrentDateAndTime = $convertedCurrentDateAndTime"
	set timeDeltaInSeconds [expr $convertedCurrentDateAndTime - $convertedFileDateAndTime]
	#puts "timeDeltaInSeconds          = $timeDeltaInSeconds"
	if { $timeDeltaInSeconds <= 180 } {
		set timeValid 1
	} else {
		set timeValid 0
	}
	
	return $timeValid
}


#dumpXMLUpdatedFile 1
#dumpXMLUpdatedFile 2
#dumpMETAXMLUpdatedFile



#############################################################################
# proc dumpXMLUpdatedFile {currentDNS}
#############################################################################
#Dumps the XML output file for the DNS of concern.
proc dumpXMLUpdatedFile {currentDNS} {

	#Availabilities.
	variable availability1
	variable availability2
	variable availabilityMeta
	#DNS characteristics
	variable DNSName1
	variable DateAndTime1
	variable NoPublications1
	variable ExceptionText1
	variable DNSName2
	variable DateAndTime2
	variable NoPublications2
	variable ExceptionText2
	variable excludedXMLUpdates
	variable ouputUpdatedXMLFile1
	variable ouputUpdatedXMLFile2
	variable availabilityInfo1
	variable availabilityInfo2
	#DNS dynamic properties
	variable interventionsStartDNS1
	variable interventionsLengthDNS1
	variable interventionsDescDNS1
	variable incidentDNS1
	variable interventionsStartDNS2
	variable interventionsLengthDNS2
	variable interventionsDescDNS2
	variable incidentDNS2
	#local properties
	variable currentXSDateTime
	
	set currentFile ""
	set id		""
	set fullName	""
	set shortname	""
	set availabilityDesc	""
	set dataDesc		""
	set interventionsStart	""
	set interventionsLength	""
	set interventionsDesc	""
	set incident		""
	set availability	""
	set timeStamp		""
	set availabilityInfo	""
	set noPublications	""
	
	#First determine if we are talking about the 1st or 2nd DNS.

	if {$currentDNS == 1} {
		#Primary DNS
		set interventionsStart $interventionsStartDNS1
		set interventionsLength $interventionsLengthDNS1
		set interventionsDesc $interventionsDescDNS1
		set incident $incidentDNS1
		set currentFile 	$ouputUpdatedXMLFile1
		set id			"DIPDNS1"
		set fullName		"Primary DIP Name Server (DNS) availability."
		set shortname		"DIP DNS1"
		set availabilityDesc	"DIPDNS1 services are fully available if the primary DNS is responding to incoming requests correctly."
		set dataDesc		"Total publications on primary DIP Name Server."
		set availability $availability1
		set availabilityInfo $availabilityInfo1
		set noPublications $NoPublications1
	} else {
		#Secondary DNS
		set interventionsStart $interventionsStartDNS2
		set interventionsLength $interventionsLengthDNS2
		set interventionsDesc $interventionsDescDNS2
		set incident $incidentDNS2 	
		set currentFile 	$ouputUpdatedXMLFile2
		set id			"DIPDNS2"
		set fullName		"Backup DIP Name Server (DNS) availability."
		set shortname		"DIP DNS2"
		set availabilityDesc	"DIPDNS2 services are fully available if the backup DNS is responding to incoming requests correctly."
		set dataDesc		"Total publications on backup DIP Name Server."
		set availability $availability2
		set availabilityInfo $availabilityInfo2
		set noPublications $NoPublications2
	}
	
	#Now that we have set the parameters, simply dump it into the XML file.
	
	set outputFile [open $currentFile w]
	puts -nonewline $outputFile "<?xml version=\"1.0\" encoding=\"utf-8\"?>
<serviceupdate xmlns=\"http://sls.cern.ch/SLS/XML/update\">
	<id>$id</id>
	<availabilitydesc>$availabilityDesc</availabilitydesc>
	<availability>$availability</availability>
	<timestamp>$currentXSDateTime</timestamp>
	<availabilityinfo>$availabilityInfo</availabilityinfo>
	<data>
		<numericvalue priority=\"low\" name=\"Publications\" desc=\"$dataDesc\" timestamp=\"$currentXSDateTime\">$noPublications</numericvalue>
	</data>"
	if { [string match "" $interventionsDesc] != 1 } { 
		puts -nonewline $outputFile "
	<!-- Optionnal stuff -->
	<interventions>
		<intervention start=\"$interventionsStart\" length=\"$interventionsLength\">$interventionsDesc</intervention>
	</interventions>"
	} else {
		#No additionnal stuff to add.	
	}

	if { [string match "" $incident] != 1 } { 
		puts -nonewline $outputFile "
	<!-- Optionnal stuff -->
	<incidents>
		<incident>$incident</incident>
	</incidents>"
	} else {
		#No additionnal stuff to add.	
	}

	#Just close the root tag.
	puts -nonewline $outputFile "
</serviceupdate>
"
}

#############################################################################
# proc dumpMETAXMLUpdatedFile {}
#############################################################################
#Dumps the XML output file for the DNS of concern.
proc dumpMETAXMLUpdatedFile {} {
	#Availabilities.
	variable availabilityMeta
	#DNS characteristics
	variable currentXSDateTime
	variable ouputUpdatedXMLFileMeta
	variable availabilityInfoMeta
	variable NoPublications1
	variable NoPublications2

	variable metaInterventionsStart
	variable metaInterventionsLength
	variable metaInterventionsDesc
	variable incidentMeta
	
	set id			"DIP"
	set fullName		"Cascaded DIP Name Servers availability"
	set shortname		"DIP Name Servers"
	set availabilityDesc	"DIP Name Servers services are fully available if both primary and backup DIP Name Servers are up and running."
	set dataDesc		"Total publications on DIP Name Servers"
	set metaPublications [ expr { $NoPublications1 + $NoPublications2} ]

	
	#Now simply dump it into the XML file.
	
	set outputFile [open $ouputUpdatedXMLFileMeta w]
	puts -nonewline $outputFile "<?xml version=\"1.0\" encoding=\"utf-8\"?>
<serviceupdate xmlns=\"http://sls.cern.ch/SLS/XML/update\">
	<id>DIP</id>
	<availabilitydesc>$availabilityDesc</availabilitydesc>
	<availability>$availabilityMeta</availability>
	<timestamp>$currentXSDateTime</timestamp>
	<availabilityinfo>$availabilityInfoMeta</availabilityinfo>
	<data>
		<numericvalue priority=\"low\" name=\"Publications\" desc=\"$dataDesc\" timestamp=\"$currentXSDateTime\">$metaPublications</numericvalue>
	</data>"
	
	if { [string match "" $metaInterventionsDesc] != 1 } { 
		puts -nonewline $outputFile "
	<!-- Optionnal stuff -->
	<interventions>
		<intervention start=\"$metaInterventionsStart\" length=\"$metaInterventionsLength\">$metaInterventionsDesc</intervention>
	</interventions>"
	} else {
		#No additionnal stuff to add.	
	}

	if { [string match "" $incidentMeta] != 1 } { 
		puts -nonewline $outputFile "
	<!-- Optionnal stuff -->
	<incidents>
		<incident>$incidentMeta</incident>
	</incidents>"
	} else {
		#No additionnal stuff to add.	
	}

	#Just close the root tag.
	puts -nonewline $outputFile "
</serviceupdate>
"
}


#############################################################################
# proc checkSendSMS {}
#############################################################################
#Dumps the XML output file for the DNS of concern.
proc checkSendSMS {} {

	#SMS and mail processing.
	variable inputFilePrimary
	variable inputFileSecondary

	variable SMSList
	variable EmergencySMSList

	#Availabilities.
	variable availability1
	variable availability2
	variable availabilityMeta

	#puts "availability1    = $availability1"
	#puts "availability2    = $availability2"
	#puts "availabilityMeta = $availabilityMeta"


	if { $availabilityMeta == -1 } {
		#Send SMS and mail to configured list
		foreach token $EmergencySMSList {
			set setString "mail -s \"All DIP DNS are DOWN !!!\" $token < \"/dev/null\""
			puts $setString
			if { [catch {eval exec $setString} result ] } {
				puts $result
			}
		}

	} else {

	#check availability of DNS1, if down ,send SMS/emails.
	if { $availability1 < 100 } {
		#Send SMS and mail to configured list
		foreach token $SMSList {
			set setString "mail -s \"DIPDNS1 is having problems\" $token < $inputFilePrimary"
			puts $setString
			if { [catch {eval exec $setString} result ] } {
				puts $result
			}
		}

	} 
	
	#check availability of DNS2, if down ,send SMS/emails.
	if { $availability2 < 100 } {
		#Send SMS and mail to configured list
		foreach token $SMSList {
			set setString "mail -s \"DIPDNS2 is having problems\" $token < $inputFileSecondary"
			puts $setString
			if { [catch {eval exec $setString} result ] } {
				puts $result
			}
		}

	}
	}
	
}

#############################################################################
# MAIN PROGRAM
#############################################################################


#Read the properties file
readInputPropertyFile

#Parse the output of the DNS checkers.
readInputDataFiles

#Now we get all the information about the DIP DNS accessibility, compute the global accessibilities.
set availability1    [computeAvalability $DNSName1 $DateAndTime1 $ExceptionText1 $NoPublications1]
set availability2    [computeAvalability $DNSName2 $DateAndTime2 $ExceptionText2 $NoPublications2]
set availabilityMeta [computeAvalabilityMeta $availability1 $availability2]

if {$DNS1NotAvailable == 1 && $DNS2NotAvailable == 1} {
	#Do nothing as nothing is available.
} elseif { $DNS1NotAvailable == 1 && $DNS2NotAvailable == 0} {
	#DNS2 status is available.
	dumpXMLUpdatedFile 2
	dumpMETAXMLUpdatedFile
} elseif { $DNS1NotAvailable == 0 && $DNS2NotAvailable == 1} {
	#DNS1 status is available.
	dumpXMLUpdatedFile 1
	dumpMETAXMLUpdatedFile
} else {
	#All available
	dumpXMLUpdatedFile 1
	dumpXMLUpdatedFile 2
	dumpMETAXMLUpdatedFile
}

#Check whether something went wrong and send SMS.
checkSendSMS

#puts "\nFinished."

#############################################################################
# END OF MAIN PROGRAM
#############################################################################

