# What is DIP ?

**DIP** is a communication system which allows relatively small amounts of soft real-time data to be exchanged between very loosely coupled heterogeneous systems.
These systems do not need very low latency. The data is assumed to be mostly summarised data rather than low-level parameters from the individual systems, i.e. cooling plant status rather than the opening level of a particular valve.

DIP is currently only supported on Windows 64-bits and Linux 64-bits. Older releases exist for 32-bits platforms.
**There is no support for Mac OSX**.

The DIP service is composed of :

* A Central Name Server that provides the list of available publications
* A C++ and a Java API (Application Programming Interface) that allows to publish and receive information.
* A WinCC OA extension (WinCC OA API Manager) that allows to publish and receive DIP data in WinCC OA.
* A LabVIEW extension that allows to publish and receive DIP data in NI LabVIEW.

More information on the [DIP Homepage](http://cern.ch/dip)

# Developer's corner

## How to build the DIP C++ API with Conan and CMake

* Install python3 and cmake (v 3.16 or later)

* Setup a Conan virtual env

```bash
#if required
pip3 install virtualenv
python3 -m venv virtualenv conan-venv
source conan-venv/bin/activate
conan create .
```

* Deploy the Conan package with

```bash
conan upload
```




## How to Release

DIP uses Git Flow to manage its development and release cycles.

The Maven Gitflow plugin offers support to issue new versions.
Simply execute :

    mvn gitflow:release-start
    
This will register a new release branch, using the version number you specify on all active DIP modules.

Your Git sandbox will automatically be switched to this new version on a local release branch of the same name.
You can refine your release, publish the branch if you wish to trigger rebuilds.
When the release is ready, finish it with :

    mvn -P basepom,all gitflow:release-finish

The result will be tagged, and the CI proccess will automatically rebuild and deploy this latest version.
