package cern.dip.Browser;

import cern.dip.*;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import javax.swing.tree.*;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.ToolTipManager;

import java.awt.event.MouseEvent;
import java.awt.Color;

import java.awt.Cursor;
import java.awt.Frame;
import java.io.IOException;
import java.io.InputStream;

class RootNode extends ToolTipTreeNode{
	private static final long serialVersionUID = 1L;

	public RootNode(){
	    super("",0);
    }
}

/**
*Enables us to distinguish between a leaf and nodes higher up the tree.
* Leaf nodes represent publications.
* */
class PublisherNode extends ToolTipTreeNode{

	private static final long serialVersionUID = 1L;
	/**
     *The name of the publication this node represents.
     * */
	private String publisherName;


	PublisherNode(String pubName, Object usrObject){
        super((String)usrObject,0);
		publisherName = pubName;
    }

    /**
     * @return the name of the publication this node represents.
     * */
    public String getPublisherName(){
		return publisherName;
    }
}

/**
*The main frame containing the publication tree. Clicking on a leaf node (publication)
* in the tree will open a DipSubscriptionView window - in which the publication
* values are displayed.
* */
public class DipMainFrame extends JFrame implements TreeSelectionListener{
	private static final long serialVersionUID = 1L;
	private DipBrowser dipBrowser = null;
    private DipFactory m_dipFactory = null;
    private String pathSeperator = "[/=(.)]";
    private JScrollPane nameSpaceScrollPanel = new JScrollPane();
    private JTree nameSpaceTree = new JTree(){
		private static final long serialVersionUID = 1L;

		public String getToolTipText(MouseEvent evt) {
            if (getRowForLocation(evt.getX(), evt.getY()) == -1)
              return null;
            TreePath curPath = getPathForLocation(evt.getX(), evt.getY());
            if (((ToolTipTreeNode) curPath.getLastPathComponent()).getToolTipText().equals("0")){
            	return null;
            }else{
            	return ((ToolTipTreeNode) curPath.getLastPathComponent()).getToolTipText()+" publications";
            }
          }
        };
    ;
    
    private static String s_browserVersion = "?.?";
    
    private JPanel m_jPanel = new JPanel();
    private static JTextField s_pubCountField = new JTextField();
    private static boolean s_selectionAllowed = true;
    private static int s_pubNamesLength = 0;
    public static final String DEFAULTDNSSTRING = "dipnstn1,dipnstn2";
    private JPanel jPanelDNS = new JPanel();
    private JTextField dipDNSField = new JTextField();
    private JButton DNSSelectButton = new JButton(">>");
    private JLabel DNSLabel = new JLabel(" DIPNS ");
    private String DNS;
    private HashMap<String, DipSubscriptionView> m_pubNameToSubscriptionViewMap = new HashMap<String, DipSubscriptionView>();
    private boolean m_connectUponStartup = false;
    
    private static Properties s_dipBrowserProps;
        
    
    static{
    	s_dipBrowserProps = new Properties();
    	InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream("dipbrowser.properties");
    	try{
    	  s_dipBrowserProps.load(is);
    	}catch(IOException ioe){
    		throw new RuntimeException("Could not initiate DIP browser - DIP properties not found on classpath");
    	}
    	s_browserVersion = s_dipBrowserProps.getProperty("dipVersion", "Unknown");
    	
    }
	/**
	 * @param value
	 */
	public static void setSelectionAllowed(boolean value) {

		if (value)
		s_pubCountField.setText("Found as many as " +s_pubNamesLength + " publications");
		s_selectionAllowed = value;
	}

	public String getDNS(){
		return DNS;
	}
	
	public String getPathSeparators(){
		return pathSeperator;
	}
	
	public static boolean getSelectionAllowed() {
		return (s_selectionAllowed);
	}

    /** Creates new form JFrame */
    public DipMainFrame(DipFactory df) {
    	DNS = System.getProperty("jnlp.dipns.hosts", System.getProperty("dipns.hosts",DEFAULTDNSSTRING));
    	// If we must prompt for a DIP NS, do not connect immediately upon startup
    	m_connectUponStartup = (!Boolean.parseBoolean(System.getProperty("jnlp.dipns.prompt", System.getProperty("dipns.prompt","true"))));

        setVisible(true);
        m_dipFactory = df;
        dipBrowser = m_dipFactory.createDipBrowser();

        nameSpaceTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        nameSpaceTree.addTreeSelectionListener(this);
        nameSpaceTree.setToolTipText("");
        nameSpaceTree.setModel(null);
        ToolTipManager.sharedInstance().setInitialDelay(0);
        
        initGUI();
        pack();
        setVisible(true);
        
        if(m_connectUponStartup) {
        	setDipnsHostsAndConnect(DNS);
        }
    }

	/**
	 * @param pubName	The non -splitted publication string.
	 * @param parent	Representes the direct parent.
	 * @param index		The recursive "pass" identifier.
	 * @param path		The splitted publication strings.
	 */
	private void buildPath(String pubName, ToolTipTreeNode parent, int index, String[] path){
		String nextStep = path[index];
		parent.iterNum();
		if (index == (path.length - 1)){
			// insert the pubNode
			parent.add(new PublisherNode(pubName, nextStep));
            return;
        }

		Enumeration<?> it = parent.children();
        while (it.hasMoreElements()){
        	ToolTipTreeNode child = (ToolTipTreeNode)it.nextElement();
            String childName = (String)child.getUserObject();
            if (childName.equals(nextStep)){
				buildPath(pubName, child, index + 1, path);
                return;
            }
        }
        // No matching child node in the tree - make one
        ToolTipTreeNode child = new ToolTipTreeNode(nextStep,0);
        parent.add(child);
		buildPath(pubName, child, index + 1, path);
    }

	private void updatePublicationTree(){
       	this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		String [] pubNames = dipBrowser.getPublications("*");
		//Here is sorted the String:
		Arrays.sort( pubNames );
		buildTree(pubNames);
        this.setCursor(Cursor.getDefaultCursor());
		s_pubNamesLength = pubNames.length;
		s_pubCountField.setText("Found as many as " + s_pubNamesLength + " publications");
    }

	private void buildTree(String [] pubNames){
        //JButton refreshRender = new JButton("Refresh");
		RootNode top = new RootNode();

        for (int i = 0; i < pubNames.length; i++){
			String [] path = pubNames[i].split(pathSeperator);
			buildPath(pubNames[i], top, 0, path);
        }

        DefaultTreeModel treeModel = new DefaultTreeModel(top);
		nameSpaceTree.setModel(treeModel);
    }

    public void valueChanged(TreeSelectionEvent e) {
    	ToolTipTreeNode node = (ToolTipTreeNode) nameSpaceTree.getLastSelectedPathComponent();
        if (node == null){
			return;
			
        }

        if (node.getClass() == RootNode.class){
			updatePublicationTree();
            return;
        }
    	else if (node.getClass() == PublisherNode.class){
			if (s_selectionAllowed) {
					if(m_pubNameToSubscriptionViewMap.containsKey(((PublisherNode)node).getPublisherName())){
						final DipSubscriptionView dsw = (DipSubscriptionView)(m_pubNameToSubscriptionViewMap.get(((PublisherNode)node).getPublisherName()));
						if(dsw.isShowing()){
							dsw.setState(Frame.NORMAL);
							java.awt.EventQueue.invokeLater(new Runnable() {
                                                            @Override
                                                            public void run() {
                                                                dsw.toFront();
                                                                dsw.repaint();
                                                            }
                                                        });
						}else{
							DipSubscriptionView newDsw = new DipSubscriptionView (m_dipFactory, ((PublisherNode)node).getPublisherName(),this); 
							m_pubNameToSubscriptionViewMap.put(((PublisherNode)node).getPublisherName(), newDsw);
							new Thread(newDsw).start();						
						}
					}else{
						DipSubscriptionView dsw = new DipSubscriptionView (m_dipFactory, ((PublisherNode)node).getPublisherName(),this); 
						m_pubNameToSubscriptionViewMap.put(((PublisherNode)node).getPublisherName(), dsw);
						new Thread(dsw).start();						
					}
			} else {
				s_pubCountField.setText("Please wait...");
			}
			return;
    	}
}

    /** This method is called from within the constructor to initialize the form. */
    private void initGUI() {
    	this.setTitle("DIP Browser " + s_browserVersion);
		addWindowListener(
            new java.awt.event.WindowAdapter() {
                public void windowClosing(java.awt.event.WindowEvent evt) {
                    exitForm(evt);
                }
            });

        getContentPane().add(m_jPanel, java.awt.BorderLayout.SOUTH);
        getContentPane().add(nameSpaceScrollPanel, java.awt.BorderLayout.CENTER);
        getContentPane().add(jPanelDNS, java.awt.BorderLayout.NORTH);
        nameSpaceScrollPanel.getViewport().add(nameSpaceTree);
        nameSpaceScrollPanel.setEnabled(false);
        nameSpaceTree.setBounds(new java.awt.Rectangle(229,187,74,72));
        nameSpaceTree.setMaximumSize(new java.awt.Dimension(600, 72));
        nameSpaceTree.setMinimumSize(new java.awt.Dimension(400, 0));
        nameSpaceTree.setEnabled(false);
        s_pubCountField.setEditable(false);
        dipDNSField.setEditable(true);
        dipDNSField.setText(DNS);
        dipDNSField.addKeyListener(new KeyListener(){
            public void keyPressed(KeyEvent e) {
            	if(e.getKeyCode() == KeyEvent.VK_ENTER){
            		DNSSelectButton.doClick();
            	}
            }

            public void keyReleased(KeyEvent e) {
            }
            
            public void keyTyped(KeyEvent e) {
            }

        });
        
        m_jPanel.setLayout(new java.awt.GridLayout(1, 2));
        m_jPanel.add(s_pubCountField);
        jPanelDNS.setLayout(new java.awt.BorderLayout());
        jPanelDNS.add(dipDNSField,java.awt.BorderLayout.CENTER);
        jPanelDNS.add(DNSSelectButton,java.awt.BorderLayout.EAST);
        jPanelDNS.add(DNSLabel,java.awt.BorderLayout.WEST);
        setResizable(true);
        nameSpaceScrollPanel.setMaximumSize(new java.awt.Dimension(300,32767));
        nameSpaceScrollPanel.setMinimumSize(new java.awt.Dimension(200,22));
        nameSpaceScrollPanel.setPreferredSize(new java.awt.Dimension(250,363));
        DNSSelectButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
           	    setDipnsHostsAndConnect(dipDNSField.getText());
            }

        });
        if(!m_connectUponStartup){
		dipDNSField.selectAll();
		dipDNSField.setFocusable(true);
		dipDNSField.requestFocus();
        }
    }
    
	private void setDipnsHostsAndConnect(String dipnsHosts) {
		DNS = dipnsHosts;
    	m_dipFactory.setDNSNode(DNS);
    	nameSpaceScrollPanel.setEnabled(true);
    	DNSSelectButton.setEnabled(false);
    	dipDNSField.setEnabled(false);
    	nameSpaceTree.setEnabled(true);
    	updatePublicationTree();
	}

    /** Exit the Application */
    private void exitForm(WindowEvent evt) {
    	System.out.println("Closing the browser");
        System.exit(0);
    }
}
