package cern.dip.Browser;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import dim.DimClient;
import dim.DimCurrentInfo;
import dim.DimErrorHandler;
import dim.DimInfo;

class DimServerInfo extends DimInfo
{
	private String server;
	private String[] clients;
	
	DimServerInfo(String name)
	{
		super(name + "/DipType", -1);
	}
	
	public void infoHandler()
	{
		server = DimClient.getServerName();
	}

	public String[] getClients() {
		return clients;
	}

	public String getServer() {
		return server;
	}
}

public class DipServerInfoView extends JFrame implements Runnable{

	private static final long serialVersionUID = 1L;
	private String publication;
	private String DNS;
	private JButton updateInfoButton = new JButton();
	private JTextField serverNameField = new JTextField();
	private JTextField lastUpdateField = new JTextField();
	private JTextArea clientsTextArea = new JTextArea();
	private JPanel serverInfoPanel = new JPanel();
	private JPanel clientsInfoPanel = new JPanel();
	private JLabel clientsLabel = new JLabel();
    private static final String DEFAULTDNS1 = "DIPNS1";
    private static final String DEFAULTDNS2 = "DIPNS2";
//    private String serverName = null;
	
    public void setNotConnected(){
    		Calendar cal = Calendar.getInstance();
	    	SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss");
    		updateInfoButton.setEnabled(true);
			serverNameField.setText("NOT CONNECTED");
			lastUpdateField.setText(sdf.format(cal.getTime()));
			clientsTextArea.setText("NOT CONNECTED");    	
			clientsLabel.setText(" Clients connected: 0");
			updateInfoButton.setEnabled(false);
    }
    
	public boolean updateInfo(){
		String serverName = null;
		Vector pubsList=new Vector();
		int j=0;
	    Calendar cal = Calendar.getInstance();
	    SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss");

	    clientsTextArea.setText("<< Getting Info ... >>");
	    updateInfoButton.setEnabled(false);
		DimServerInfo dsi = new DimServerInfo(publication);
		try{
			int i = 0;
			while (dsi.getServer() == null && i < 30){
				Thread.sleep(100);
				i++;
			}
		} catch (Exception e) {
			System.err.println("Error: " + e.getMessage());
		}
		serverName = dsi.getServer();
    	if(serverName == null){
    		updateInfoButton.setEnabled(true);
   			serverNameField.setText("NOT CONNECTED");
   			lastUpdateField.setText(sdf.format(cal.getTime()));
   			clientsTextArea.setText("NOT CONNECTED");    	
   			clientsLabel.setText(" Clients connected: 0");
   			return false;
   		}
   		DimCurrentInfo dimClients = new DimCurrentInfo(serverName.substring(0,serverName.indexOf('@')) + "/CLIENT_LIST",-1);
   		String[] clients = (dimClients.getString()).split("\\|");

   		for(j = 0;j < clients.length;j++){
   			if (!clients[j].equals("")){
   				pubsList.add(clients[j]);
   			}
   		}
    	serverNameField.setText(serverName);
    	lastUpdateField.setText(sdf.format(cal.getTime()));
    	clientsTextArea.setText("");
    	for(int i=0;i<pubsList.size();i++){
        	clientsTextArea.setText(clientsTextArea.getText() + (String)pubsList.get(i) + "\n");    		
    	}
    	clientsLabel.setText(" Clients connected: " + pubsList.size());
    	updateInfoButton.setEnabled(true);
    	return true;
	}
	
    public DipServerInfoView(String pubName, String dipDNS) {
		publication = pubName;
		DNS = dipDNS;
    }
    
    
    public void run(){

    	DimErrorHandler erid = new DimErrorHandler()
        {
            public void errorHandler(int severity, int code, String msg)
            {
                System.out.println("Error: "+msg+" sev: "+severity);
            }
        }; 
        DimClient.addErrorHandler(erid);
		setTitle("Server Info for " + publication);
		serverInfoPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		
		JLabel label1 = new JLabel(" Server: ");
		serverInfoPanel.add(label1, c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 0;
		serverNameField.setBorder(null);
        serverNameField.setEditable(false);
		serverInfoPanel.add(serverNameField, c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 1;
		JLabel label2 = new JLabel(" Last Update: ");
		serverInfoPanel.add(label2, c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 1;
		lastUpdateField.setEditable(false);
		lastUpdateField.setBorder(null);
		serverInfoPanel.add(lastUpdateField, c);
		
		clientsLabel = new JLabel(" Clients connected: 0");
		clientsLabel.setHorizontalAlignment(JLabel.CENTER);
		clientsInfoPanel.setLayout(new java.awt.BorderLayout());
		clientsInfoPanel.add(clientsLabel,java.awt.BorderLayout.NORTH);
		
        clientsTextArea.setEditable(false);
        clientsTextArea.setText("<< Getting Info ... >>");
		clientsInfoPanel.add(clientsTextArea,java.awt.BorderLayout.CENTER);
		
		getContentPane().setLayout(new java.awt.BorderLayout());
		getContentPane().add(serverInfoPanel, java.awt.BorderLayout.NORTH);
		getContentPane().add(clientsInfoPanel, java.awt.BorderLayout.CENTER);
		updateInfoButton.setText("UPDATE");
		getContentPane().add(updateInfoButton, java.awt.BorderLayout.SOUTH);
        
        updateInfoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	updateInfo();
            }
        });
		setSize(new java.awt.Dimension(400, 300));
        setResizable(true);
//        pack();
        this.setVisible(true);
//        System.err.println(DimClient.getDnsNode());
        String dns_env = null;
        if(DNS.equals(DipMainFrame.DEFAULTDNSSTRING)){
        	
        	java.lang.Process p;
        	try{
                if(System.getProperty("os.name").toLowerCase().startsWith("windows")){
                	p = java.lang.Runtime.getRuntime().exec("CMD /c \"echo %DIPNS%\"");
                }else{
                	p = java.lang.Runtime.getRuntime().exec("printenv DIPNS");
                }
        		p.waitFor();
        		java.io.InputStream is = null ;
        		is = p.getInputStream() ;
        	    java.io.BufferedInputStream bis = new java.io.BufferedInputStream(is);

        	    byte [] ab = new byte[64000];
        	    int nTotalBytesRead = 0;
        	    while(nTotalBytesRead < 64000) {
        	    	int nBytesRead = bis.read(ab, nTotalBytesRead, 64000 - nTotalBytesRead);
        	    	if(nBytesRead < 0){
        	    		break ;
        	    	}else{
        	    		nTotalBytesRead += nBytesRead;            	       
        	    	}
        	    }

        	    dns_env = new String(ab, 0, nTotalBytesRead) ;
        	    if(dns_env.startsWith("%DIPNS%")){
        	    	dns_env = null;
        	    }
        	}catch(Exception e){
        		   
        	}
        	if(dns_env == null){
        		DNS = DEFAULTDNS1;
        		DimClient.setDnsNode(DNS, 2506);
        		if(!updateInfo()){
        			DNS = DEFAULTDNS2;
        			DimClient.setDnsNode(DNS, 2506);
        			updateInfo();
        		}
        	}else{
       			DNS = dns_env.trim();
        		DimClient.setDnsNode(DNS, 2506);
        		updateInfo();
        	}
    	}else{
    		DimClient.setDnsNode(DNS, 2506);
    		updateInfo();
    	}
//        System.out.println(DNS);
        

    }
}
