<?xml version="1.0"?>

<!-- Sample stylesheet which renders JIRA's RSS (XML) output in a HTML format similar to the {jiraissues} macro in Confluence. See navigator-rss.jsp for how to apply this automatically. -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

	<xsl:param name="url" select="'undefined.jira.url'"/>

	<xsl:param name="title_type" select="'Type'"/>
	<xsl:param name="title_key" select="'Key'"/>
	<xsl:param name="title_summary" select="'Summary'"/>
	<xsl:param name="title_status" select="'Status'"/>
	<xsl:param name="title_resolution" select="'Res'"/>
	<xsl:param name="title_description" select="'Description'"/>
    
    <xsl:param name="project_name"/>
	<xsl:param name="version"/>
    <xsl:param name="project_url"/>
   

	<xsl:variable name="lowercase" select="'abcdefghijklmnopqrstuvwxyz'" />
	<xsl:variable name="uppercase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ '" />

	<xsl:template match="/rss/channel">
		<html>
			<head>
              <script><![CDATA[
                  function HTMLDecode(s){
                    s = s.replace(new RegExp(/&lt;/gi), "<");
                    s = s.replace(new RegExp(/&gt;/gi), ">");
                    s = s.replace(new RegExp(/&amp;/gi), "&");
                    return s;
               }]]>
               </script>
               <link rel="stylesheet" href="http://wikis.cern.ch/s/1627/45/11/_/styles/combined.css?spaceKey=EN" type="text/css"/>
               <link rel="stylesheet" type="text/css" href="http://wikis.cern.ch/s/1627/45/1/_/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/hmenu/src/skin-longhorn.css"/>
               <link rel="stylesheet" type="text/css" href="http://wikis.cern.ch/plugins/builder/layout-resources.action?layoutId=ENDEPARTMENT&amp;key=EN&amp;resource=combinedcss&amp;hash=-59677564s16274511_"/>

				<title><xsl:value-of select="$project_name"/>&#160;<xsl:value-of select="$version"/> Change Log</title>
			</head>
			<body style="margin: 20px; background-color: white;">
                <h1><xsl:value-of select="$project_name"/>&#160;<xsl:value-of select="$version"/> Change Log</h1>
                <noscript><b>WARNING</b> : Javascript is required for correct rendering of the <b>description</b> column.</noscript>
                <p>
				<table class="grid" cellspacing="0" width="100%">
					<caption style="text-align: left; font-weight: bold">
							<a href="{$project_url}"><xsl:value-of select="$project_name"/> version <xsl:value-of select="$version"/></a>
							<span class="smalltext"> (<xsl:value-of select="count(item)"/> issues)</span>
						</caption>
					<tbody>
						<tr>
							<th style="text-align: left;"> <xsl:value-of select="$title_type"/> </th>
							<th style="text-align: left;"> <xsl:value-of select="$title_key"/> </th>
							<th style="text-align: left;"> <xsl:value-of select="$title_summary"/> </th> 
							<th style="text-align: left;"> <xsl:value-of select="$title_description"/> </th>
						</tr>
						<xsl:for-each select="item">
							<tr>
								<xsl:attribute name="class">
									<xsl:choose>
										<xsl:when test="position() mod 2 = 0">rowNormal</xsl:when>
										<xsl:otherwise>rowAlternate</xsl:otherwise>
									</xsl:choose>
								</xsl:attribute>
								<xsl:apply-templates select="type"/>
								<xsl:apply-templates select="key"/>
								<xsl:apply-templates select="summary"/>
								<xsl:apply-templates select="description"/>
							</tr>
						</xsl:for-each>

					</tbody>
				</table>
                </p>
			</body>
		</html>
	</xsl:template>


	<xsl:template match="type">
		<td valign="top" nowrap="true">
			<a href="{../link}"><xsl:value-of select="."/></a>
		</td>
	</xsl:template>

	<xsl:template match="key">
		<td valign="top" nowrap="true">
			<b><a href="{../link}"><xsl:value-of select="."/></a></b>
		</td>
	</xsl:template>

	<xsl:template match="summary">
		<td valign="top">
			<b><a href="{../link}"><xsl:value-of select="."/></a></b>
		</td>
	</xsl:template>

	<xsl:template match="description">
		<td valign="top">
           <xsl:attribute name="id">td<xsl:number level="any"/></xsl:attribute>
           <xsl:value-of select="."/>
			 <script>td<xsl:number level="any"/><![CDATA[.innerHTML=HTMLDecode(]]>td<xsl:number level="any"/><![CDATA[.innerHTML);]]></script>
		</td>
	</xsl:template>

	<xsl:template match="resolution">
		<td valign="top" nowrap="true">
			<b><xsl:value-of select="translate(., $lowercase, $uppercase)"/></b>
		</td>
	</xsl:template>



</xsl:stylesheet>

