#!/bin/ash

# This script renames the dimbridge process to a custom name
# so that bridges can be uniquely identified in the process table
# and restarted individually.



# if the env variable BRIDGE_NAME is specified, create a symbolic link

set -e

PRCSS=dimbridge

if [[ "$BRIDGE_NAME" ]]; then
    echo "Bridge Name is defined, aliasing the process name..."
    if [[ ! -e /$BRIDGE_NAME ]]; then
        cd / && ln -s /usr/local/bin/dimbridge $BRIDGE_NAME
    fi
    PRCSS="$BRIDGE_NAME"
    
    # if this doesn't run, the bridge name does not correspond to
    # an existing configuration
    # if [ -f /etc/dip-bridges/$BRIDGE_NAME/env.props ]; then
    #   echo "... and sourcing the corresponding environment"
    #   export eval `cat /etc/dip-bridges/$BRIDGE_NAME/env.props`
    # fi
fi

if [[ -z "$FROM_NODE" ]] || [[ -z "$TO_NODE" ]]; then
    echo "Please specify the bridge FROM and TO nodes"
    exit 1
fi


# if `docker run` first argument start with `--` the user is passing launcher arguments
if [[ $# -lt 1 ]] || [[ "$1" == "--"* ]]; then
  echo "Starting up the bridge with $PRCSS"
  env
  set -x
  su-exec dim:1000 ./$PRCSS "$@"
fi

echo "Bridge not configured, starting default command..."
su-exec dim:1000 "$@"
