#ifndef SERVICE_NAME_MAPPER_HPP

#define SERVICE_NAME_MAPPER_HPP

#include <string>
#include <map>
#include "boost/regex.hpp"

using namespace std;
using namespace boost;

/**
 * This mapper class allows to match service names and support replacement
 * format strings complying with the ECMA-262 standard.
 */
class ServiceNameMapper{
   private:
     /**
      * Matches a regular expression (e.g. "dip/acc/LHC/Beam/(.*)" ) to
      * a replace pattern ( e.g. "dip/proxy/acc/LHC/Beam/$1" )
      */
     map<regex*, string> m_matchToReplacePattern; 
     
     /**
      * Keeps track of any DIM Service being actively forwarded. 
      */
     map<string,string> m_sourceToDestinationServiceMap;
   
   public:
     string findServiceDestinationName(const string source);
     
     string getDestinationName(const string source);
     
     bool registerSourceToDestinationServiceMapping(const string source, const string dest);
     
     bool registerSourceToDestinationServiceMapping(const string source);
     
     bool unregisterSourceToDestinationServiceMapping(const string source);
     
     /**
      * Use variable environments to populate the regular expression mapping
      * table.
      */
     void configure(map<string, string> matchToReplacePattern);
};

#endif /* SERVICE_NAME_MAPPER_HPP */