#!/usr/bin/env python

##############################
# This script converts an everyconfig YAML fileset into a list of
# environment variables.
# Inspired by https://gist.github.com/SteveBenner/588fff3e54636f3d8297 without the Ruby
# Depends on everyconfig and visitor packages
#

import sys
from visitor import Visitor
from everyconfig import everyconfig

if len(sys.argv) < 2:
    raise RuntimeError("Please provide an input YAML path")

varsYaml = everyconfig(sys.argv[1]) 

if varsYaml is None:
    raise RuntimeError("Could not parse input YAML path")

class EnvEncoder(Visitor):
    def __init__(self):
        self.parentKey = ""
    
    def visit_str(self, node):
        print "export "+self.parentKey+"="+node
    
    def visit_int(self, node):
        return self.visit(str(node))
    
    def visit_bool(self, node):
        return self.visit(str(node))

    def visit_list(self, node):
        if len(node) > 0:
            baseParentKey = self.parentKey+("" if self.parentKey =="" else "_")
            print "export "+baseParentKey+"COUNT="+str(len(node))
            for index, item in enumerate(node):
                self.parentKey = baseParentKey+str(index)
                self.visit(item)

    def visit_dict(self, node):
        prevParent = self.parentKey
        for key, value in sorted(node.items()):
            self.parentKey = prevParent+("" if prevParent =="" else "_")+key.upper()
            if type(value) == str:
                print "export "+self.parentKey+" = "+value
            else:
                self.visit(value)
        # Pop the last known key
        self.parentKey = prevParent
       
EnvEncoder().visit(varsYaml)
