#include <gtest/gtest.h>
#include <stdio.h>
#include <map>
#include "ServiceNameMapper.hpp"
#include "boost/regex.hpp"
#include <string>
#include "test_dim_bridge_main.hpp"

using namespace std;


TEST(dimBridgeUnitTests, ServiceNameMapperTest) {
  ServiceNameMapper mapper;
  map<string, string> mappings;
  
  mappings["dip/acc/(.*?)/Beam/(.*)"] = "dip/proxy/acc/$1/Beam/$2";
  mappings["dip/acc/(.*?)/Status/(.*)" ] = "dip/proxy/acc/$1/Status/Flag/$2";
  
  mapper.configure(mappings);
  
  EXPECT_EQ("dip/proxy/acc/LHC/Beam/P1", mapper.getDestinationName("dip/acc/LHC/Beam/P1"));
  EXPECT_EQ("dip/proxy/acc/PS/Beam/P1", mapper.getDestinationName("dip/acc/PS/Beam/P1"));
  EXPECT_EQ("dip/proxy/acc/LHC/Status/Flag/P1", mapper.getDestinationName("dip/acc/LHC/Status/P1"));
  EXPECT_EQ("dip/proxy/acc/SPS/Status/Flag/P2", mapper.getDestinationName("dip/acc/SPS/Status/P2"));
  
  mapper.registerSourceToDestinationServiceMapping("dip/acc/LHC/Beam/P1");
  EXPECT_EQ("dip/proxy/acc/LHC/Beam/P1", mapper.findServiceDestinationName("dip/acc/LHC/Beam/P1"));
  EXPECT_TRUE(mapper.findServiceDestinationName("dip/proxy/acc/LHC/Beam/P1").empty());
  
  
  {
    // Unregister the source service from the map....
    EXPECT_TRUE(mapper.unregisterSourceToDestinationServiceMapping("dip/acc/LHC/Beam/P1"));
    // Then ensure the source service is gone (along with the destination)
    EXPECT_TRUE(mapper.findServiceDestinationName("dip/acc/LHC/Beam/P1").empty());
  }
}



//void print_captures(const std::string& regx, const std::string& text)
//{
//   boost::regex e(regx);
//   boost::smatch what;
//   cout << "Expression:  \"" << regx << "\"\n";
//   cout << "Text:        \"" << text << "\"\n";
//   if(boost::regex_match(text, what, e, boost::match_extra))
//   {
//      unsigned i, j;
//      cout << "** Match found **\n   Sub-Expressions:\n";
//      for(i = 0; i < what.size(); ++i)
//         cout << "      $" << i << " = \"" << what[i] << "\"\n";
//      cout << "   Captures:\n";
//      for(i = 0; i < what.size(); ++i)
//      {
//         cout << "      $" << i << " = {";
//         for(j = 0; j < what.captures(i).size(); ++j)
//         {
//            if(j)
//               cout << ", ";
//            else
//               cout << " ";
//            cout << "\"" << what.captures(i)[j] << "\"";
//         }
//         cout << " }\n";
//      }
//   }
//   else
//   {
//      cout << "** No Match found **\n";
//   }
//}
//
//
//TEST(dimBridgeUnitTests, DISABLED_BoostRegexTest) {
//  print_captures("(([[:lower:]]+)|([[:upper:]]+))+", "aBBcccDDDDDeeeeeeee");
//   print_captures("(.*)bar|(.*)bah", "abcbar");
//   print_captures("(.*)bar|(.*)bah", "abcbah");
//   print_captures("^(?:(\\w+)|(?>\\W+))*$",
//      "now is the time for all good men to come to the aid of the party");
//      
//  EXPECT_EQ(1, 1);
//}



