package cern.dip.libjdim;

import dim.*;
import java.util.Date;
import junit.framework.*;

/**
 * This class serves both as a complete test of the Dim package, as well as a show case for the capabilities of DIM.
 * This class is still under construction.
 * @author B. Copy
 */
public class TestDimServer extends TestCase
{

    public void testServer(){
        DimServer.setDnsNode("localhost");
        System.out.println("Using DIM DNS : "+DimServer.getDnsNode()+" "+DimServer.getDnsPort());
	    DimErrorHandler eid = new DimErrorHandler()
         {
            public void errorHandler(int severity, int code, String msg)
            {
              if(code == DIMSVCDUPLC)
               System.out.println("Service already declared");
              System.out.println("Error: "+msg+" sev: "+severity);
            }
         };
        DimServer.addErrorHandler(eid);
        
        DimExitHandler exid = new DimExitHandler()
         {
             public void exitHandler(int code)
             {
                 System.out.println("Exit: "+code);
             }
         };
	    DimServer.addExitHandler(exid);
        
        DimCommand cid = new DimCommand("TestCommand","I")
         {
             public void commandHandler()
             {
                 int i;
                 System.out.println("Received "+getInt());
                 String[] list = DimServer.getClientServices();
                 System.out.println("Services: ");
                 for(i = 0; i < list.length; i++)
                     System.out.println(list[i]);
             }
         };
        int i = 0;
        DimService sid1 = new DimService("TEST_IT_INT",i);
        DimService cid1 = new DimService("TEST_IT_STR","hello hello");
        
        int myArr[] = {0,1,2,3,4,5};
        DimService aa = new DimService("TEST_IT_ARRAY",myArr);
        
        DimService testMix = new DimService();
        testMix.setInt(123);
        testMix.setString(20, "hello");
        testMix.setDouble(3.5);
        testMix.setIntArray(new int[] {2,3,4});
        testMix.setLong((long)123456);
        testMix.setStringArray(new String[] {"aaa","bbb","ccc"});
        testMix.setName("TEST_MIX");
        testMix.updateService();
        DimServer.start("TestJDim");

         int count = 0;
        while(count < 2)
        {
            DimTimer.sleep(1);
            sid1.setQuality(i);
            sid1.setTimestamp(new Date());
            sid1.updateService(++i);
            int myArr1[] = {0,1,2,3,4,5,6,7,8};
            aa.updateService(myArr1);
            testMix.setInt(456);
            testMix.setString(20, "hello world");
            testMix.setDouble(6.5);
            testMix.setIntArray(new int[] {6,7,8});
            testMix.setLong((long)12345678);
            testMix.setStringArray(new String[] {"aaa1","bbb1","ccc1"});
            testMix.updateService();
            count++;
            System.out.println("Count "+count);
        }
        
 }
 
 public void testClient(){
    DimTimer.sleep(3);
    DimClient.setDnsNode("localhost");
    System.out.println("Client Using DIM DNS : "+DimClient.getDnsNode()+" "+DimClient.getDnsPort()+" and server "+ DimClient.getServerName() );
    DimClient.addErrorHandler(new DimErrorHandler()
         {
            public void errorHandler(int severity, int code, String msg)
            {
              System.out.println("Error: "+msg+" sev: "+severity);
            }
         });
    
    DimInfo tt2 = new DimInfo("TEST_IT_INT2", -1)
    {
        public void infoHandler()
        {
            int i;
            System.out.println("Received "+getInt());
            System.out.println("Quality "+getQuality());
            System.out.println("Timestamp "+getTimestamp());
            String[] list = DimClient.getServerServices();
            System.out.println("service count : "+list.length);
            System.out.println("Services: ");
            for(i = 0; i < list.length; i++)
                System.out.println(list[i]);
        }
    };
    
    int i = 0;
    DimService sid1 = new DimService("TEST_IT_INT2",i);
    sid1.setQuality(i);
    sid1.setTimestamp(new Date());
    sid1.updateService(++i);
    
    try{
    Thread.sleep(2000);
    } catch(InterruptedException ie){
        fail("Thread was interrupted");
    }
 }

    

}
