#include "Dip.h"
#include <cstring>
#include <string>
#include <iostream>
#include <sstream>
#include <vector>
#include "DIPVersion.h"
#include "DipSubscription.h"
#include <unistd.h>

#define _GLIBCXX_USE_CXX11_ABI 0

using namespace std;

/**
* Simple test case for the requestUpdate bug (ENS-26231)
* */

string TOPIC = "dip/test/force-republish";

class ErrHandler:public DipPublicationErrorHandler{
	void handleException(DipPublication* publication, DipException& ex){
		printf("DIP Error upon publication : %s", ex.what());
	}
};

void runServer(){
 	DipPublication *pub;
	DipData *pubData;
	DipTimestamp aTimeStamp;

	string serverName = "force-republish-server";
	DipFactory *dip = Dip::create(serverName.c_str());
	ErrHandler errorHandler;
    pub =dip->createDipPublication(TOPIC.c_str(), &errorHandler);
	
    DipBool boolValue=true;

    while(true){
        pubData= dip->createDipData();
        pubData->insert(boolValue);
        DipTimestamp time;
         
        cout<< "Posting BOOLEAN data to pub "<< pub->getTopicName() << endl;
        pub->send(pubData, time);
        sleep(1);
    }
    
	cout<<"DIP server lifetime has expired, leaving... "<<endl;
    dip->destroyDipPublication(pub);   
}


/**
* Handler for connect/disconnect/data reception events
* Nested class
* */
class ClientDataHandler:public DipSubscriptionListener{
public:
	ClientDataHandler(){};

	void handleMessage(DipSubscription *subscription, DipData &message){
		cout<<"Client : Received data from "<<subscription->getTopicName()<<endl;
	}

	void connected(DipSubscription *subscription) {
		cout << "\nClient : Publication source  " << subscription->getTopicName()<< " available\n";
	}

	void disconnected(DipSubscription *arg0, char *arg1) { }

	void handleException(DipSubscription* subscription, DipException& ex){
		cout << "Client : Subs "<<subscription->getTopicName()<<" has error "<< ex.what()<<endl;
   }
};

void runClient(){
 	DipSubscription *sub;
	DipData *data;
	ClientDataHandler client;

	string clientName = "force-republish-client";
	DipFactory *dip = Dip::create(clientName.c_str());
    sub = dip->createDipSubscription(TOPIC.c_str(), &client);
	
    while(true){
        sleep(2);
        cout<<".";
        sub->requestUpdate();
    }
    
	cout<<"DIP client stopped, leaving... "<<endl;
    dip->destroyDipSubscription(sub);   
}


int main(const int argc, const char** argv) {
   
   cout<<" Using "<<DipVersion::getDipVersion() <<endl;


   if(argc ==1 || (argc > 1 && strcmp(argv[1], "server")  == 0)){
       cout<<" Running the server" <<endl;
       runServer();
   }else if(argc > 1 && strcmp(argv[1], "client")  == 0){
       cout<<" Running the client" <<endl;
       runClient();
   }
   
	
    cout<<"Done."<<endl;
	return 0;
}

