#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include "Options.h"
#include "Dip.h"
#include "DipDimFactory.h"
#include <iostream>
#include "dis.hxx"

using namespace std;

	class DipErrorHandler : public DimErrorHandler
    {
		void errorHandler(int severity, int code, char *msg)
        {
			Dip::handler->handleException(severity, code, msg);
		}
        public:
            DipErrorHandler() {
				DimServer::addErrorHandler(this);
				DimClient::addErrorHandler(this);
			}
		};
	class ExitHandler : public DimExitHandler
		{
			void exitHandler(int code)
            {
            }
            public:
            ExitHandler() {
				DimServer::addExitHandler(this);
			}
        };

//class for catching DIM exceptions
class DipErrHandler:public DipDimErrorHandler{
	void handleException(int severity, int code, char *msg){
        string msgString = "DIM message: ";
        msgString += msg;
		log4cplus::Logger dimErrorLogger(log4cplus::Logger::getInstance("dip.system.dimerror"));
		switch(severity)
		{
			case 0:
				dimErrorLogger.log(log4cplus::INFO_LOG_LEVEL, LOG4CPLUS_TEXT(msgString));
				break;
			case 1:
				dimErrorLogger.log(log4cplus::WARN_LOG_LEVEL, LOG4CPLUS_TEXT(msgString));
			    break;
			case 2:
				dimErrorLogger.log(log4cplus::ERROR_LOG_LEVEL, LOG4CPLUS_TEXT(msgString));
				break;
			case 3:
				dimErrorLogger.log(log4cplus::FATAL_LOG_LEVEL, LOG4CPLUS_TEXT(msgString));
				break;
		}
	}
};

void Dip::addDipDimErrorHandler(DipDimErrorHandler * errHandler)
{
	handler = errHandler;
	DipErrorHandler *erh = new DipErrorHandler();

	ExitHandler *exh = new ExitHandler();
}

DipFactory *Dip::create(const char *username, const char* logPropertiesFilePath, const int loggingConfigWatchInterval) 
{
	// TODO check DNS is running
	if (theFactory == NULL){
		theFactory = new DipDimFactory(username, logPropertiesFilePath, loggingConfigWatchInterval);
		LOG4CPLUS_INFO(log4cplus::Logger::getInstance("dip.system.factory"),LOG4CPLUS_TEXT("Instantiated new DIP factory '" << username<< "'"));
//		m_diagnosticPTree.put("dip.system.factory.username", username);
	}else{
		((DipDimFactory *)theFactory)->setName(username);
		LOG4CPLUS_INFO(log4cplus::Logger::getInstance("dip.system.factory"),LOG4CPLUS_TEXT("Updated DIP factory name to '" << username<< "'"));
//		m_diagnosticPTree.put("dip.system.factory.username", username);
	}
	DipErrHandler* der = new DipErrHandler;
	addDipDimErrorHandler(der);

	return theFactory;
}

DipFactory *Dip::create() 
{
	if (theFactory == NULL){
		theFactory = new DipDimFactory();
		log4cplus::Logger::getInstance("dip.system.factory").log(log4cplus::INFO_LOG_LEVEL,LOG4CPLUS_TEXT("Instantiated new DIP factory"));
//		m_diagnosticPTree.put("dip.system.factory.username", "__anonymous__");
	}
	DipErrHandler* der = new DipErrHandler;
	addDipDimErrorHandler(der);

	return theFactory;
}


void Dip::shutdown() {
	log4cplus::Logger::getInstance("dip.system.factory").log(log4cplus::INFO_LOG_LEVEL,LOG4CPLUS_TEXT("Now shutting down DIP factory"));
	if (theFactory != NULL){
		delete theFactory;
	}
	theFactory = NULL;
	log4cplus::Logger::getInstance("dip.system.factory").log(log4cplus::INFO_LOG_LEVEL,LOG4CPLUS_TEXT("DIP factory shutdown complete"));
}


DipFactory * Dip::theFactory = NULL;
DipDimErrorHandler * Dip::handler = NULL;
