#include "DipDataClient.h"
#include "DipDimData.h"
#include "DipDataService.h"
#include "dic.hxx"

#include "DipLogger.h"

using namespace std;

DipDataClient::DipDataClient(const std::string & dipPubName, DipDataObjectListener & datalistener):
DimStampedInfo((dipPubName + DipDataService::dipDataServiceEnding).c_str(), const_cast<char *>(DipTypeClient::noLink)),
	dataObjectListener(datalistener){
	pubName = dipPubName;
}

void DipDataClient::infoHandler(){
	unsigned int dataByteSize = this->getSize();
	//Get the raw DIM data pointer.
	void * dipData = this->getData();

	if ((dataByteSize -1) == strlen(DipTypeClient::noLink)){
		const char * dimString = getString();
		// DIP-6 : c.f. bug description
        if (strcmp(dimString, DipTypeClient::noLink) == 0){
			// no meaningful data
			return;
		}
	}

	// OK at this point we should have something usefull to work with
	
	if (!dipDataExtractor.isFormatDefined()){
		// we are receiving data - but format is unknown - this is NOT GOOD
		// TODO should queue it 
		LOG4CPLUS_ERROR(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX, pubName), LOG4CPLUS_TEXT("DipDataClient::infoHandler() - recvd DIP data but format not known"));
		return;
	}
	if (!dipDataExtractor.isFormatValid()){
		// we are receiving data - but format is unknown - this is NOT GOOD
		// TODO should queue it 
		LOG4CPLUS_ERROR(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX, pubName), LOG4CPLUS_TEXT("DipDataClient::infoHandler() - recvd DIP data but format not known"));
		return;
	}


    // DIP-6 : c.f. bug description
	if(dipData == NULL){
		LOG4CPLUS_WARN(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX, pubName), LOG4CPLUS_TEXT("DipDataClient::infoHandler() - Received DIP data NULL pointer, dropping DIP packet"));
		return;
	}

	int quality = this->getQuality();
	if (quality == DIP_QUALITY_UNINITIALIZED){
		// should never be uninit since we should only get this far if data has been published.
		LOG4CPLUS_ERROR(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX, pubName), LOG4CPLUS_TEXT("DipDataClient::infoHandler() - data quality is uninitialized"));
	}
	DipLong time_sec = (DipLong)getTimestamp() * (DipLong)1000000000L;
	DipLong time_nsec = (DipLong)getTimestampMillisecs();
	DipLong time_ns = time_sec + time_nsec;
	DipData * dataObject = NULL;
	try{
		dataObject = dipDataExtractor.extractDipDataObject(dipData, dataByteSize, (DipQuality)quality, time_ns);
	} catch (DipException &de){
		LOG4CPLUS_ERROR(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX, pubName), LOG4CPLUS_TEXT("DipDataClient::infoHandler() - Failed to extract data due to exception : "<< de.what()));
		return;
	}
	dataObjectListener.dipDataReceived(*dataObject);
	delete dataObject;
}

