#include "DipDataExtractor.h"


void DipDataExtractor::setDipDataFormatValidity(int valid){
	dipDataFormatValid = valid;
}

void DipDataExtractor::defineDipDataFormat(const std::string &format){// throws DipException
	if (isFormatDefined()){
		clearDipServiceMap();
		// This is not good - the format has been defined once, now its changing
// CG		throw DipException(std::string("Fatal - DIP data format changed - was ") + dipDataFormatDescription + " now " + format);
	}
	dipDataFormatDescription = format;
	dipDataFormatValid = 1;
	buildDipServiceMap();
#if ( defined (linux) || defined (WIN32) )
	//Do not use buildDimFormatString().
#else
		buildDimFormatString();
#endif
}
		
DipDimData *DipDataExtractor::extractDipData(const void * data, const unsigned int byteSize) const{
	if ((dipDataFormatDescription.size() == 0) || (dipDataServiceFields.size() == 0)){
		throw DipException(std::string("DIP data structure not defined"));		
	}
	
	unsigned int startOfQualityString = startOfVarSection;
	DipDimData * dipData = new DipDimData();
	int nItems, nBytes;
	char *pdata, *pvalues;

	for (unsigned int i = 0; i < dipDataServiceFields.size(); i++){
		DimServiceField &fieldInfo = *dipDataServiceFields[i];
		try{
			int fieldType = fieldInfo.dipField.fieldType;

			pdata = (char *)data;
			pvalues = (char *)data + fieldInfo.serviceOffset;
			nItems = fieldInfo.dipField.cardinality;
			if(fieldType >= TYPE_ARRAY_VAR_START)
			{
				pvalues = (char *)data + startOfQualityString;
				nItems = *(int *)pvalues;
				startOfQualityString += sizeof(int);
				pvalues += sizeof(int);
			}
			switch(fieldType){
				case TYPE_STRING:{
					const char * value = DataBlock::readString(data, byteSize, startOfQualityString);
					dipData->insert(value, fieldInfo.dipField.fieldName.c_str());
					startOfQualityString += strlen(value) + 1;
					break;
				}
				case TYPE_STRING_ARRAY:
				case TYPE_STRING_ARRAY_VAR:{
					const char ** values = new const char*[nItems];
					for (unsigned int sind = 0; sind < nItems; sind++){
						values[sind] = DataBlock::readString(pdata, byteSize, startOfQualityString);
						startOfQualityString += (strlen(values[sind]) + 1);
					}
					dipData->insert(values, nItems, fieldInfo.dipField.fieldName.c_str());
					delete []values;
					break;
				}
				case TYPE_BOOLEAN_ARRAY_VAR:
				case TYPE_BYTE_ARRAY_VAR:
				case TYPE_DOUBLE_ARRAY_VAR:
				case TYPE_FLOAT_ARRAY_VAR:
				case TYPE_INT_ARRAY_VAR:
				case TYPE_LONG_ARRAY_VAR:
				case TYPE_SHORT_ARRAY_VAR:{
					nBytes = nItems * DipDimData::getTypeSize((DipDataType)fieldType);
					startOfQualityString += nBytes;
				}
				case TYPE_BOOLEAN:
				case TYPE_BOOLEAN_ARRAY:
				case TYPE_BYTE:
				case TYPE_BYTE_ARRAY:
				case TYPE_DOUBLE:
				case TYPE_DOUBLE_ARRAY:
				case TYPE_FLOAT:
				case TYPE_FLOAT_ARRAY:
				case TYPE_INT:
				case TYPE_INT_ARRAY:
				case TYPE_LONG:
				case TYPE_LONG_ARRAY:
				case TYPE_SHORT:
				case TYPE_SHORT_ARRAY:{
					dipData->insert(pvalues, 
									fieldInfo.dipField.fieldName.c_str(), 
									fieldInfo.dipField.fieldType,
									nItems);
					break;
				}	
				default: throw DipInternalError(std::string("Unknown type"));
			}
		} catch (DipException &de){
			std::string message("DipDataExtractor::extractDipData() - extract field ");
			message += fieldInfo.dipField.fieldName + std::string(" failed: ") + de.what();
			throw DipInternalError(message);
		} catch (std::runtime_error &re){
			std::string message("DipDataExtractor::extractDipData() - extract field ");
			message += fieldInfo.dipField.fieldName + std::string(" failed: ") + re.what();
			throw DipInternalError(message);
		}
	}

	if (startOfQualityString > byteSize){
		char message[100];
		sprintf(message,"DipDataExtractor::extractDipData() - position of quality string (%d) > sizeof data(%d)", startOfQualityString, byteSize);
		throw DipInternalError(message);
	}

	try{
		const char * qualReason = DataBlock::readString(data, byteSize, startOfQualityString);
		dipData->setQualityString(qualReason);
	} catch (std::runtime_error & de){
		DataBlock remainingData((byteSize - startOfQualityString) + 2);
		char nullTerm = 0;
		remainingData.write(((char *)data)+startOfQualityString,  (byteSize - startOfQualityString) + 1);
		remainingData.write(&nullTerm, 1);
		std::string message("DipDataExtractor::extractDipData() - get quality reason failed because ");
		message=+ de.what();
		message=+ (char *)remainingData.getDataBlock();
		throw DipInternalError(message);
	}
		
	return dipData;
}
	
	

	/**
	 * Used to extract both the Dip data and the Dip attributes 
	 * */
DipDimData * DipDataExtractor::extractDipDataObject(const void * data, unsigned int dataByteSize, DipQuality qual, longlong time_ns) const{
		DipDimData *dipData = extractDipData(data, dataByteSize);

		// OK now get and set the DIPdata attributes.
		DipTimestamp time;
		time.setNanos(time_ns);
		dipData->setDipTime(time);
		dipData->setDataQuality(qual);
		return dipData;
	}

