#include "DipDataInserter.h"
#include "DipDataExtractor.h"
#include "DipDataType.h"
#include <memory.h>


bool DipDataInserter::verifySerialization = false;
const SString  DipDataInserter::emptyQualityReasonMessage = "";

void DipDataInserter::setDipStructureString(const DipDimData & data){
	std::string structure="";
	int offsetIntoService = 0;    			// where the first byte of the next field in the service is to start
	const std::map<const std::string *, Field *, StringLess> & fields = data.getFields();
	std::map<const std::string *, Field *, StringLess>::const_iterator it = fields.begin();
	std::vector<Field *> stringFields;  	// used to store string base fields untill we are ready to process them

	while (it != fields.end()){
			Field &field = *(it->second);
			DipDataType fieldType = field.fieldType;
			if ((fieldType == TYPE_STRING) || (fieldType == TYPE_STRING_ARRAY) || (fieldType >=  TYPE_ARRAY_VAR_START)){
				// leave strings till later
				stringFields.push_back(&field);
			} else{
				char buff[50];
				sprintf(buff,"|%d|%d|%d|",offsetIntoService, field.byteSize, field.fieldType);
				// <tagName>|<byteOffset>|numBytes|<enumeratedType>|
				structure += field.fieldName + buff;
				offsetIntoService += field.byteSize;
			}
			it++;
		}
	
		// OK, by this time the fixed part of the structure is defined - now for the var part (strings)
		for (unsigned int i = 0; i < stringFields.size(); i++){
			Field &field = *stringFields[i];
			char buff[50];
			sprintf(buff,"|%d|%d|%d|",i, field.cardinality, field.fieldType);
			//<tagName>|<byteOffset>(sequence)|numBytes(cardinality)|<enumeratedType>|
			structure += field.fieldName + buff;		 
		}
	
		dipDataFormatDescription = structure;
	}





void DipDataInserter::writeDipObjectIntoDataBlock(const DipDimData & dipData){// throws DipException, TypeMismatch
	if (dipDataServiceFields.size() == 0){
		setDipStructureString(dipData);
		buildDipServiceMap();
#if ( defined (linux) || defined (WIN32) )
	//Do not use buildDimFormatString().
#else
		buildDimFormatString();
#endif
	}
		
	if (dipDataServiceFields.size() != dipData.size()){
	  std::string message = "Data structure changed";
	  throw TypeMismatch(message);
	}
		
	DataBlock *newData = new DataBlock(dipData.dataByteSize());
	int qualityReasonWritePosition = startOfVarSection;
	const std::map<const std::string *, Field *, StringLess> & dataObjectFields = dipData.getFields();	

	for (unsigned int i = 0; i < dipDataServiceFields.size(); i++){
		DimServiceField &dataServiceField = *dipDataServiceFields[i];
		const std::string & dataServiceFieldName = dataServiceField.dipField.fieldName;
		std::map<const std::string *, Field *, StringLess>::const_iterator it = dataObjectFields.find(&dataServiceFieldName);

		/// Make sure expected structure has not changed.
		if (it == dataObjectFields.end()){
		  std::string message = "Data structure changed, missing field " + dataServiceFieldName; 
		  throw TypeMismatch(message);
		}
		Field & dataObjectField = *(it->second);
		if ((dataServiceField.dipField.fieldType != dataObjectField.fieldType) ||
			(dataServiceField.dipField.cardinality != dataObjectField.cardinality)){
		  std::string message = "Data structure changed, changed field type for " + dataServiceFieldName;
		  throw TypeMismatch(message);
		}


		if ((dataObjectField.fieldType == TYPE_STRING) ||
			(dataObjectField.fieldType == TYPE_STRING_ARRAY) || 
			(dataObjectField.fieldType >=  TYPE_ARRAY_VAR_START)){
			// Inc ONLY if we are handling strings
			newData->write(dataObjectField.data, dataObjectField.byteSize, qualityReasonWritePosition);
			qualityReasonWritePosition += dataObjectField.byteSize;
			
		}else{
			newData->write(dataObjectField.data, dataObjectField.byteSize, dataServiceField.serviceOffset);
		}
	}

	// Now write an empty quality reason string
	newData->write(emptyQualityReasonMessage.getString(), emptyQualityReasonMessage.strLen()+1, qualityReasonWritePosition);


#ifdef _DEBUG_NOTNOW
	// verfy data is packed OK
	DipDataExtractor serviceDataExractor;
	serviceDataExractor.defineDipDataFormat(this->getDipDataFormat());
	unsigned int size = newData->getNumBytesStored();
	const void * rawData = newData->read(size, 0);
	std::auto_ptr<DipDimData>extractedData(serviceDataExractor.extractDipData(rawData, size));
	const std::map<const std::string *, Field *, StringLess> & extractedDataMap = extractedData->getFields();
	const std::map<const std::string *, Field *, StringLess> & originalDataMap = dipData.getFields();
	if (extractedDataMap.size() != originalDataMap.size()){
		throw DipInternalError(std::string("Data not serialized correctly:- wrong number of fields\n"));
	}
	std::map<const std::string *, Field *, StringLess>::const_iterator it = originalDataMap.begin();
	while (it != originalDataMap.end()){
		Field * originalField = it->second;
		std::map<const std::string *, Field *, StringLess>::const_iterator targetIt = extractedDataMap.find(&(originalField->fieldName));
		Field * extractedField = targetIt->second;
		if (extractedField == NULL){
			throw DipInternalError(std::string("Data not serialized correctly: missing field\n"));
		}
		if (originalField->equals(*extractedField) == false){
			throw DipInternalError(std::string("Data not serialized correctly:- field value/type/cardinality is wrong"));
		}
		it++;
	}
#endif
		
	startOfQualityReasonString = qualityReasonWritePosition;
	if (data){
		delete data;
	}
	data = newData;
}




void DipDataInserter::setQualityExplainationString(const std::string &reason){
	data->write(reason.c_str(), reason.size()+1, startOfQualityReasonString);
}

