#include "DipDataService.h"
#include "DipDataInserter.h"
#include "DipDimFactory.h"
#include <iostream>
#include "dim.h"


using namespace std;


const std::string DipDataService::dipDataServiceEnding("/DipData");





DipDataService::DipDataService(const std::string & dipName, DipDataInserter * dataSerializer, const DipData & data, const DipQuality dipQuality, const SString qualityReason ):
    dimService(NULL),dipDataSerializer(dataSerializer)
{
	unsigned int dataSize, fixedSize;
	const void * rawData = dipDataSerializer->getDataBlock(dataSize);
	fixedSize = dipDataSerializer->getFixedSize();
	
#if defined (linux) || defined (WIN32)
	char dataFormat[50000];
	memset(dataFormat,'\0',sizeof(dataFormat));
	sprintf(dataFormat, "C:%d;C", (fixedSize));
//cout << "Creating "<< const_cast<char *>((dipName + dipDataServiceEnding).c_str()) << " format: " << dataFormat << endl;
	dimService = new DimService(const_cast<char *>((dipName + dipDataServiceEnding).c_str()),dataFormat,(void *)rawData,dataSize);
	dimService->setData(const_cast<void *>(rawData), dataSize);
	LOG4CPLUS_DEBUG(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX, dipName),LOG4CPLUS_TEXT("Registered DIP data service '"<<dipName<<"' with format '"<<dataFormat<<"'"));
#else
	if (dipDataSerializer->getDimDataFormat().length() >= MAX_NAME) {
		string msg = "Specified DIP Data format descriptor exceeds DIM maximum length.";
		LOG4CPLUS_FATAL(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX, dipName),LOG4CPLUS_TEXT(msg));
		throw DipException(msg);
	}
	dimService = new DimService(const_cast<char *>((dipName + dipDataServiceEnding).c_str()),(char *)(dipDataSerializer->getDimDataFormat().c_str()),(void *)data,dataSize);
	LOG4CPLUS_DEBUG(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX, dipName),LOG4CPLUS_TEXT("Registered DIP data service '"<<dipName<<"' with format '"<<((char *)(dipDataSerializer->getDimDataFormat().c_str()))<<"'"));
#endif
	
	time_ns = data.extractDipTime().getAsNanos();
	dataQuality = dipQuality;
	reasonForQuality = qualityReason;
	sendDipData();
}

DipDataService::~DipDataService(){
	if (dimService){
		delete dimService;
	}

	if (dipDataSerializer){
		delete dipDataSerializer;
	}
}

void DipDataService::sendDipData(){
	dimService->setQuality(dataQuality);
	long dimTsSec = long(time_ns/1000000000L);
	long dimTsmilliSec = long(time_ns%1000000000L);
	dimService->setTimestamp(dimTsSec, dimTsmilliSec);

	unsigned int dataSize;
	const void * data = dipDataSerializer->getDataBlock(dataSize);
	dimService->setData(const_cast<void *>(data), dataSize);
	dimService->updateService();
	LOG4CPLUS_DEBUG(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimService->getName()),LOG4CPLUS_TEXT("sent DIP data with timestamp "<<dimTsSec<<" sec "<<dimTsmilliSec<<" msec"));
}

void DipDataService::sendQualityIfChanged(DipQuality quality, const SString &reason){
	if ((quality != dataQuality) || 
		(reason != reasonForQuality)){
		reasonForQuality = reason;
		dataQuality = quality;
		dipDataSerializer->setQualityExplainationString(reasonForQuality);
		LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimService->getName()),LOG4CPLUS_TEXT("sendQualityIfChanged()- resending DIP data with new quality '"<< dataQuality <<"' (reason : '"<<reasonForQuality.getString()<<"')"));

		sendDipData();
	}
}

void DipDataService::setQualityUncertain(const SString &reason){
	sendQualityIfChanged(DIP_QUALITY_UNCERTAIN, reason);
}

void DipDataService::setQualityBad(const SString &reason){
	sendQualityIfChanged(DIP_QUALITY_BAD, reason);
}

void DipDataService::resendToClient(int clientID){
	dimService->setQuality(dataQuality);
	dimService->setTimestamp(int(time_ns/1000000000L), int(time_ns%1000000000L));
	LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimService->getName()),LOG4CPLUS_TEXT("resendToClient()- resending DIP data to client '"<< clientID<<"'"));
	dimService->selectiveUpdateService(&clientID);
}

void DipDataService::updateDipData(const DipData & data){
	updateDipData(data, DIP_QUALITY_GOOD, "");
}

void DipDataService::updateDipData(const DipData & data, const DipQuality quality, const SString qualityReason){
	time_ns = data.extractDipTime().getAsNanos();
	dataQuality = quality;
	reasonForQuality = qualityReason;
	dipDataSerializer->writeDipObjectIntoDataBlock((DipDimData &)data);
	dipDataSerializer->setQualityExplainationString(reasonForQuality);
	LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimService->getName()),LOG4CPLUS_TEXT("Updating DIP data to all clients - quality '"<< dataQuality <<"' (reason : '"<<reasonForQuality.getString()<<"')"));
	sendDipData();
}