#include "DipDimBrowser.h"
#include "DipDimClient.h"
#include "DipTypeService.h"
#include "DipTypeUtil.h"
#include <vector>

DipBrowser::~DipBrowser() {}


const char ** DipDimBrowser::getPublications(const char * pattern, unsigned int &noPublications)
{	
//	DTRACE(("In DipDimBrowser::getPublications\n"));
	destroyPubList();
	noPublications = 0;

	SString wildCard((pattern == NULL) ? "*":pattern);
	wildCard += DipTypeService::dipTypeServiceEnding;
	std::vector<std::string> actualPublicationNames; // DIM appears to be reporting false pubs - this is used to hold the real ones

	// get dim names - sort out real from fake - convert to DIP names
	int reportedNoPubs = dimNameSpaceBrowser.getServices(wildCard.getString());
//	DINFO(("Found %d services matching wildcard %s\n", reportedNoPubs, wildCard.getString()));	
	int i;
	for(i = 0; i < reportedNoPubs; i++)
	{
		char * nextDimName = NULL;
		char * dimFormat = NULL;
		int dimServiceType = dimNameSpaceBrowser.getNextService(nextDimName, dimFormat);
		if (dimServiceType != DimSERVICE){
			DINFO(("Was expecting %s to be a DIM service not %d\n", nextDimName, dimServiceType));
			continue;
		}
		// convert DIM name to DIP name and store
		std::string currentDimName(nextDimName);
		unsigned int trimPoint = currentDimName.find(DipTypeService::dipTypeServiceEnding.getString());
		if (trimPoint == std::string::npos){
			//FIXME, Gcc reports :  warning: comparison is always false due to limited range of data type.
			DERROR(("dimNameSpaceBrowser.getNextService() returned %s which does not match filter %s\n", currentDimName.c_str(), wildCard.getString()));
			continue;
		}

/*
		std::string result;
		try {
			result = dimData.substr(DipVersion::getDipVersionStringLen(), dimData.size() - DipVersion::getDipVersionStringLen());
			} catch (std::out_of_range & ex) {
				DERROR(("Error occured in extractDipDataFromDimTypeServiceData : %s\n", ex.what()));
			} catch (...) {
				DERROR(("%s\n", "Error occured in extractDipDataFromDimTypeServiceData while cutting away DIP Version info."));
			}
		return (result); 

*/
		try {
			std::string result = currentDimName.substr(0,trimPoint);
			actualPublicationNames.push_back(result);
		} catch (std::out_of_range & ex) {
			DERROR(("Error occured in getPublications : %s\n", ex.what()));
		} catch (...) {
			DERROR(("%s\n", "Error occured in getPublications while cutting away DIP Version info."));
		}
	}
	
	// store DIP names
	numberOfPublications = actualPublicationNames.size();
	pubNames = new char *[numberOfPublications];
	for (i = 0; i < numberOfPublications; i++){
		int pubNameLen = actualPublicationNames[i].size() + 1;
		pubNames[i] = new char[pubNameLen];
		memcpy(pubNames[i], actualPublicationNames[i].c_str(), pubNameLen);
	//	DINFO(("%d. Found Pub name %s\n",i, pubNames[i]));
	}

	noPublications = numberOfPublications;
	DINFO(("Returning %d publication names\n", noPublications));
//	DTRACE(("Out DipDimBrowser::getPublications\n"));
	return (const char **)pubNames;
}





const char ** DipDimBrowser::getTags(const char * pub, unsigned int &noFields)
{
//	DTRACE(("In DipDimBrowser::getTags\n"));
	destroyFieldList();
	if (publicationDataExtractor){
		delete publicationDataExtractor;
		publicationDataExtractor = NULL;
	}

	SString pubName(pub ? pub : "");
	pubName += DipTypeService::dipTypeServiceEnding;
	
	DimCurrentInfo dipClient(pubName.getString(), const_cast<char *>(DipTypeClient::noLink));
	const char *dimString = dipClient.getString();
	DINFO(("Got type data %s for dim service %s\n", dimString, pubName.getString()));
	if (strcmp(DipTypeClient::noLink, dimString) == 0){
		// service does not exist
		throw DipException(std::string(pub) + " does not exist");
	}
	else {
		std::string dipDataString = DipTypeUtil::extractDipDataFromDimTypeServiceData(dimString);
		publicationDataExtractor = new DipDataExtractor();
		try{
			publicationDataExtractor->defineDipDataFormat(dipDataString);
			fieldNamesAsStringArray = publicationDataExtractor->getFieldNames(numberOfFields);
			fieldNames = new char *[numberOfFields];
			for (int i = 0; i < numberOfFields; i++){
				fieldNames[i] = const_cast<char *>(fieldNamesAsStringArray[i].c_str());
				DINFO(("%d. Found field %s\n", i, fieldNames[i]));
			}
		} catch(...){
			// if we are here then the format has not yet been defined
			numberOfFields = 0;
			fieldNames = new char *[numberOfFields];
			DINFO(("Pub %s has no fields defined\n", pub));
		}
	}
	
	noFields = numberOfFields;
	DINFO(("Returning %d field names for pub %s\n", noFields, pub));
//	DTRACE(("Out DipDimBrowser::getTags\n"));
	return (const char **)fieldNames;
}




int DipDimBrowser::getType(const char * tag)
{
//	DTRACE(("In DipDimBrowser::getType\n"));
	if (publicationDataExtractor == NULL){
		throw DipException("Usage: must first call getTags() to set the publication");
	}

	SString tagName(tag ? tag : "");
	int type = publicationDataExtractor->getFieldType(tagName);
	DINFO(("Field %s is of type %d\n", tag, type));
//	DTRACE(("Out DipDimBrowser::getType\n"));
	return type;
}



int DipDimBrowser::getSize(const char * tag)
{
//	DTRACE(("In DipDimBrowser::getSize\n"));
	if (publicationDataExtractor == NULL){
		throw DipException("Usage: must first call getTags() to set the publication");
	}
	
	SString tagName(tag ? tag : "");
	int size = publicationDataExtractor->getFieldCardinality(tagName);
	DINFO(("Field %s has carinality %d\n", tag, size));
	DTRACE(("Out DipDimBrowser::getSize\n"));
	return size;
}
	
