#include "DipDimData.h"
#include "DipException.h"



const char * DipDimData::fieldNameWhenPrimitive = "__DIP_DEFAULT__";

DipDimData::~DipDimData(){
		if (fieldNamesHolder != NULL){
			delete [] fieldNamesHolder;
		}

		std::map<const std::string *, Field *, StringLess>::iterator it = fields.begin();

		while (it != fields.end()){
			delete (it->second);
			it++;
		}
}


Field & DipDimData::retrieveField(const char * tag) const //throws BadParameter
{
	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}

	Field * field = findField(tag);
	if (field == NULL){
	  std::string message = std::string("No such field ") + tag;
	  throw BadParameter(message);
	}
	return *field;
}



Field & DipDimData::retrieveTypedField(const char * tag, const DipDataType type) const//throws BadParameter, TypeMismatch
{
	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}

	Field * field = lookForTypedField(tag, type);
	if (field == NULL){
	  std::string message = std::string("No such field ") + tag;
	  throw BadParameter(message);
	}
	return *field;
}
	

Field *DipDimData::lookForTypedField(const char *tag, const DipDataType type) const// throws TypeMismatch 
{

	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}
	Field * field = findField(tag);
	if (field != NULL){
		if (field->fieldType != type){
		  std::string message = std::string("Tag ") + tag + " is of wrong type";
		  throw TypeMismatch(message);
		}
	}
	return field;
}	
	



Field *DipDimData::lookForTypedField(const char *tag, const DipDataType type, const unsigned int cardinality)const {//throws TypeMismatch
	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}
	Field * field = lookForTypedField(tag, type);
	if (field != NULL){
//		if (field->cardinality != cardinality){
//		  std::string message = std::string("Tag ") + tag + " has wrong cardinality";
//			throw TypeMismatch(message);
//		}
	}
	return field;
}





Field * DipDimData::findField(const char *tag) const{
	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}
	std::string fieldName(tag);
	std::map<const std::string *, Field *, StringLess>::const_iterator it = fields.find(&fieldName);
	Field * field = (it == fields.end()) ? NULL : it->second;
	return field;
}




const std::string & DipDimData::extractString(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_STRING);
	return *field.convertCstringToStandardStringType();
}


const char * DipDimData::extractCString(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_STRING);
	return (const char * const)field.data;
}


DipBool	DipDimData::extractBool(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_BOOLEAN);
	return *(DipBool *) field.data;
}


DipByte DipDimData::extractByte(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_BYTE);
	return *(DipByte *)field.data;
}


DipShort DipDimData::extractShort(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_SHORT);
	return *(DipShort *)field.data;
}


DipInt DipDimData::extractInt(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_INT);
	return *(DipInt *)field.data;
}


DipFloat DipDimData::extractFloat(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_FLOAT);
	return *(DipFloat *)field.data;
}


DipDouble DipDimData::extractDouble(const char *tag) const{
Field & field = retrieveTypedField(tag, TYPE_DOUBLE);
	return *(DipDouble *)field.data;
}


DipLong	DipDimData::extractLong(const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_LONG);
	return *(DipLong *)field.data;
}


const DipBool * DipDimData::extractBoolArray(int &size,	const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_BOOLEAN_ARRAY);
	size = field.cardinality;
	return (DipBool * const) field.data;
}



const DipByte * DipDimData::extractByteArray(int &size, const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_BYTE_ARRAY);
	size = field.cardinality;
	return (DipByte *) field.data;
}


const DipShort * DipDimData::extractShortArray(int &size, const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_SHORT_ARRAY);
	size = field.cardinality;
	return (DipShort *) field.data;
}

/*
const DipInt * DipDimData::extractIntArray(int &size,	const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_INT_ARRAY);
	size = field.cardinality;
	return (DipInt *) field.data;
}
*/

const DipInt * DipDimData::extractIntArray(int &size,	const char *tag) const{
//	Field & field = retrieveTypedField(tag, TYPE_INT_ARRAY_VAR);
	char *pdata;
	Field & field = retrieveField(tag);
	pdata = (char *)field.data;
	if(field.fieldType >= TYPE_ARRAY_VAR_START)
	{
		size = *(int *)pdata;
		pdata += sizeof(int);
	}
	else
	{
		size = field.cardinality;
	}
	return (DipInt *) pdata;
}


const DipFloat * DipDimData::extractFloatArray(int &size, const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_FLOAT_ARRAY);
	size = field.cardinality;
	return (DipFloat *) field.data;
}


const DipDouble * DipDimData::extractDoubleArray(int &size,const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_DOUBLE_ARRAY);
	size = field.cardinality;
	return (DipDouble *) field.data;
}


const DipLong * DipDimData::extractLongArray(int &size,	const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_LONG_ARRAY);
	size = field.cardinality;
	return (DipLong *) field.data;
}


const std::string * DipDimData::extractStringArray(int &size,const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_STRING_ARRAY);
	size = field.cardinality;
	return field.convertCstringToStandardStringType();
}



const char **DipDimData::extractCStringArray(int &size,const char *tag) const{
	Field & field = retrieveTypedField(tag, TYPE_STRING_ARRAY);
	size = field.cardinality;
	return field.holderForCstrArray;
}



void DipDimData::insert(const void *data, const char *tag, const DipDataType type, const int cardinality){
	int tmptype;
	if (data == NULL){
		throw DipException("DipDimData::insert - caller passed NULL value");
	}
	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}
	tmptype = type;
	Field * field = findField(tag);
	if (field != NULL){
		if (field->fieldType != type){
			tmptype -= TYPE_ARRAY_VAR_START;
			if(field->fieldType != (DipDataType)tmptype){
				std::string message = std::string("Tag ") + tag + " is of wrong type";
				throw TypeMismatch(message);
			}
		}
		if (field->cardinality != cardinality){
			if(tmptype < TYPE_ARRAY_VAR_START)
			{
				std::string message = std::string("Tag ") + tag + " has wrong cardinality";
				throw TypeMismatch(message);
			}
		}
	}
	else
	{
		if((type >= TYPE_ARRAY_VAR_START) && (cardinality > 0))
			tmptype -= TYPE_ARRAY_VAR_START;
		field = new Field(tag, (DipDataType)tmptype, cardinality);
		fields[&(field->fieldName)]=field;
		fieldsChanged = true;
	}
	field->setData(data, cardinality);
}




void DipDimData::insert(const std::string &value, const char *tag){
	insert(value.c_str(), tag);
}




void DipDimData::insert(const char *address,const char *tag){
	insert((void *)address, tag, TYPE_STRING, 1);
}



void DipDimData::insert(DipBool value,	const char *tag){
	insert((void *)&value, tag, TYPE_BOOLEAN, 1);
}



void DipDimData::insert(DipByte value,const char *tag){
	insert((void *)&value, tag, TYPE_BYTE, 1);
}



void DipDimData::insert(DipShort value,	const char *tag){
	insert((void *)&value, tag, TYPE_SHORT, 1);
}



void DipDimData::insert(DipInt value, const char *tag){
	insert((void *)&value, tag, TYPE_INT, 1);
}



void DipDimData::insert(DipLong value,const char *tag){
	insert((void *)&value, tag, TYPE_LONG, 1);
}



void DipDimData::insert(DipFloat value,	const char *tag){
	insert((void *)&value, tag, TYPE_FLOAT, 1);
}



void DipDimData::insert(DipDouble value,const char *tag){
	insert((void *)&value, tag, TYPE_DOUBLE, 1);
}


/*
void DipDimData::insert(const DipBool addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_BOOLEAN_ARRAY, size);
}


void DipDimData::insert(const DipByte addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_BYTE_ARRAY, size);
}


void DipDimData::insert(const DipShort addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_SHORT_ARRAY, size);
}


void DipDimData::insert(const DipInt addr[],	int size, const char *tag){
	insert((void *)addr, tag, TYPE_INT_ARRAY, size);
}


void DipDimData::insert(const DipLong addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_LONG_ARRAY, size);
}


void DipDimData::insert(const DipFloat addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_FLOAT_ARRAY, size);
}


void DipDimData::insert(const DipDouble addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_DOUBLE_ARRAY, size);
}*/

void DipDimData::insert(const DipBool addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_BOOLEAN_ARRAY_VAR, size);
}


void DipDimData::insert(const DipByte addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_BYTE_ARRAY_VAR, size);
}


void DipDimData::insert(const DipShort addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_SHORT_ARRAY_VAR, size);
}


void DipDimData::insert(const DipInt addr[],	int size, const char *tag){
	insert((void *)addr, tag, TYPE_INT_ARRAY_VAR, size);
}


void DipDimData::insert(const DipLong addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_LONG_ARRAY_VAR, size);
}


void DipDimData::insert(const DipFloat addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_FLOAT_ARRAY_VAR, size);
}


void DipDimData::insert(const DipDouble addr[],int size, const char *tag){
	insert((void *)addr, tag, TYPE_DOUBLE_ARRAY_VAR, size);
}


void DipDimData::insert(const std::string addr[],int size,const char *tag){
	if (addr == NULL){
		throw DipException("DipDimData::insert - caller passed NULL value");
	}
	const char **strs = new const char *[size];
	for (int i = 0; i  < size; i++){
		strs[i] = addr[i].c_str();
	}
	insert(strs, size, tag);
	delete[] strs;
}



void DipDimData::insert(const char * addr[],	int size, const char *tag){
	insert((void *)addr, tag, TYPE_STRING_ARRAY_VAR, size);
}







DipDataType DipDimData::getValueType(const char *tag) const{
	Field *field = findField(tag);
	if (field == NULL){
		return TYPE_NULL;
	}
	return field->fieldType;
}




unsigned DipDimData::getValueDimension(const char *tag) const{
	Field *field = findField(tag);
	if (field == NULL){
		return 0;
	}
	return field->cardinality;
}




const char **DipDimData::getTags(int &nTags) const{
	/*there are a few fudges in here to get arround the fact this is a constant
	method - yet we are changing the objects state!
	The changes are only for book keeping purposes - the important parts of the
	state do not change
	*/
	const char *** fieldNames = const_cast<const char ***>(&fieldNamesHolder);
	if (fieldsChanged == true){
		bool & fieldsChangedMutable = const_cast<bool &>(fieldsChanged);
		fieldsChangedMutable = false;
		if (*fieldNames == NULL){
			delete [](*fieldNames);
		}

		nTags = size();
		*fieldNames = new const char *[nTags]; 
		int i = 0;
		std::map<const std::string *, Field *, StringLess>::const_iterator it = fields.begin();
		while (it != fields.end()){
			(*fieldNames)[i] = const_cast<char *>((it->first)->c_str());
			it++;
			i++;
		}
	}
	return *fieldNames;
}


	

bool DipDimData::contains(const char tag[]) const{
	if (tag == NULL){
		tag = fieldNameWhenPrimitive;
	}
	return (findField(tag) != NULL);
}



unsigned int DipDimData::size() const{
	return fields.size();
}


bool DipDimData::isEmpty() const{
	return (size() == 0);
}



DipQuality DipDimData::extractDataQuality() const{
	return dipQuality;
}


const DipTimestamp & DipDimData::extractDipTime() const{
	return dipTimestamp;
}


const std::string & DipDimData::extractReasonForQuality() const{
	return reasonForQuality;
}


int DipDimData::getTypeSize(DipDataType dipType)
{
	int size = 1;
	int type = (int) dipType;
	if (type > 90){
		type -= 100; // variable array type is *10 then +100 the enumerated value of its primitive
		type /= 10;
	}
	else if (type > 9){
		type /= 10; // array type is *10 the enumerated value of its primitive
	}
	switch(type)
	{
		case TYPE_BOOLEAN:
		case TYPE_BYTE:
		case TYPE_STRING:
			size = 1;
			break;
		case TYPE_SHORT:
			size = 2;
			break;
		case TYPE_INT:
		case TYPE_FLOAT:
			size = 4;
			break;
		case TYPE_DOUBLE:
		case TYPE_LONG:
			size = 8;
			break;
		default:
			size = 1;
			break;	
	}
	return size;
}




unsigned int DipDimData::dataByteSize() const{
	const std::map<const std::string *, Field *, StringLess> & fields = getFields();
	std::map<const std::string *, Field *, StringLess>::const_iterator it = fields.begin();
	unsigned int size = 0;
	while (it != fields.end()){
		size += it->second->byteSize;
		it++;
	}

	return size;
}
	



