#include "DipDimServer.h"
#include "DipDimFactory.h"
#include <iostream>

#include "DipLogger.h"

using namespace std;


DipDimServer::DipDimServer(const std::string & topicname, DipPublication & publication):
dimName(topicname),
dimTypeService(NULL),
dimDataService(NULL),
dimCommand(NULL)
{
	dimTypeService = new DipTypeService(topicname);
	DimServer::start(dimName.c_str());
	dimCommand = new DipUpdateCommand((dimName+"/DipUpdate").c_str(),*this);
	DimServer::start(dimName.c_str());
}

void DipDimServer::send(const DipDimData &value)
{	
	send(value, DIP_QUALITY_GOOD, "");
}
	
void DipDimServer::send(const DipDimData &value, const DipQuality quality,const SString qualityReason)
{	
	if(dimDataService == NULL)
	{
		DipDataInserter * dataSerializer = new DipDataInserter();
		dataSerializer->writeDipObjectIntoDataBlock(value);
		dataSerializer->setQualityExplainationString(qualityReason);
		/*
		 * First data send - now we know the structure of the type we can define the type service
		 * and create the data service
		 * */
		std::string dipTypeDescriptor = dataSerializer->getDipDataFormat();
		dimTypeService->setTypeFormat(dipTypeDescriptor);

		dimDataService = new DipDataService(dimName, dataSerializer, value, quality, qualityReason); // ownership of serialiser passed
		DimServer::start(dimName.c_str());
	}
	else
	{
		dimDataService->updateDipData(value, quality, qualityReason);
	}
}



	
	

void DipDimServer::resendWithNewQuality(DipQuality quality, const SString & reason)
{
	SString qualityReason = (reason.strLen() == 0) ? DipDataInserter::emptyQualityReasonMessage : reason;
	if(dimDataService == NULL)
	{
		// pass quality info through the type service
		if (quality == DIP_QUALITY_BAD){
			dimTypeService->setQualityBad(qualityReason);
		} else {
			dimTypeService->setQualityUncertain(qualityReason);
		}
	} else {
		if (quality == DIP_QUALITY_BAD){
			dimDataService->setQualityBad(qualityReason);
		} else {
			dimDataService->setQualityUncertain(qualityReason);
		}
	}
}



void DipDimServer::updateSingleClient()
{
	int clientID = DimServer::getClientId();
	LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimName),LOG4CPLUS_TEXT("client '"<<clientID<<"' requested a forced update."));

	if(dimDataService != NULL){
		dimDataService->resendToClient(clientID);
		LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimName),LOG4CPLUS_TEXT("Resent current DIP data to client '"<<clientID<<"'."));
	}
	else{
		// Pub data structure not defined - resend type info
		// TODO race hazard here if data service is begin created when this happens
		dimTypeService->resendToClient(clientID);
		LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_PUB_PREFIX,dimName),LOG4CPLUS_TEXT("No DIP data currently available, resent current DIP type descriptor to client '"<<clientID<<"'."));
	}
}
