#include "DipSyntaxEnforcer.h"

using namespace std;

//Issues a warning in case the DIP publication doesn`t start with "dip" or contains strange characters.
bool DipSyntaxEnforcer::checkPublicationName(const std::string & dipPubName, const log4cplus::Logger logger) {
	if(! (logger.isEnabledFor(log4cplus::WARN_LOG_LEVEL)) ){
		return true;
	}
	bool result = true;
	string::const_iterator begin = dipPubName.begin();
	string::const_iterator end   = dipPubName.end();
	//Special token delimiters are | / = ( ) . \n
	bool isDelimiter = false;
	for (;begin != end; begin++) {

		//Search for strange characters.
		size_t positionChar = string::npos;
		positionChar = authorizedChars.find(*begin,size_t(0));
		if (positionChar != string::npos) {
			//Ok, the character is in the authorizedChars list.
			} else {
				//Problem, the character found isn`t in the authorizedChars list.
				string messageWarning = "The non standard character or delimiter : \"";
				messageWarning += *begin;
				messageWarning += "\" has been found in pubName = ";
				messageWarning += dipPubName;

				logger.log(log4cplus::WARN_LOG_LEVEL,LOG4CPLUS_TEXT(messageWarning));
				string messageInfo = "authorizedChars characters and delimiters are = ";
				messageInfo += authorizedChars;
				logger.log(log4cplus::INFO_LOG_LEVEL,LOG4CPLUS_TEXT(messageInfo));
				result = false;
				continue;
			}

		//Check for successive delimiters.
		size_t positionDelimiter = string::npos;
		positionDelimiter = authorizedDelimiters.find(*begin,size_t(0));
		if (positionDelimiter != string::npos) {
			//This character is a delimiter. If previous was already a delimiter, report an issue.
			if (true == isDelimiter) {
				string messageWarning = "Successive delimiters detected in ";
				messageWarning += dipPubName;
				logger.log(log4cplus::WARN_LOG_LEVEL,LOG4CPLUS_TEXT(messageWarning));
				string messageInfo = "Make sure to remove unneeded delimiters in ";
				messageInfo += dipPubName;
				logger.log(log4cplus::INFO_LOG_LEVEL,LOG4CPLUS_TEXT(messageInfo));
				result = false;
				} else {
					isDelimiter = true;
				}
			} else {
				//Normal character, dont care.
				isDelimiter = false;
			}
		} //end for loop
	//Now make sure it start with the first token "dip"
	string dipString( "dip" );
	string::size_type dipLocation = dipPubName.find( dipString, 0 );
	if( dipLocation != 0 ) {
		  //Problem, the first token shall be "dip"
          LOG4CPLUS_WARN(logger,LOG4CPLUS_TEXT("The publication name must start with \"dip\" in Publication = '" << dipPubName <<"'"));
	}
	return result;
	}

const std::string DipSyntaxEnforcer::authorizedChars      = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789@|/=()._";
const std::string DipSyntaxEnforcer::authorizedDelimiters = "/=(.)";
