#include "DipTypeClient.h"
#include "DipTypeService.h"
#include "DipDimData.h"
#include "DipTypeUtil.h"

#include "DipLogger.h"


const char * DipTypeClient::noLink= "___*NoLiNk*__";



DipTypeClient::DipTypeClient(const SString & dipPubName, DipDataObjectListener & datalistener, DipDataFormatListener & formatlistener, DisconnectListener & discListener):
DimStampedInfo((dipPubName + DipTypeService::dipTypeServiceEnding).getString(),const_cast<char *>(noLink)),
dataObjectListener(datalistener),
dataFormatListener(formatlistener), 
disconnectListener(discListener),
dimServiceName(dipPubName + DipTypeService::dipTypeServiceEnding)
{
}

void DipTypeClient::infoHandler(){
	const char * dimString = getString();
//	DINFO(("DipTypeClient::infoHandler got %s from type service %s\n", dimString, dimServiceName.getString()));
	LOG4CPLUS_INFO(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX,dimServiceName),LOG4CPLUS_TEXT(" Type service got "<<dimString<<" from type service "<< dimServiceName.getString()));
		
	if (strcmp(dimString, noLink) == 0){
		disconnectListener.disconnected(*this);
	} else if (0 == (unsigned long)strlen(dimString)) {
		//Then the DIM data does not contain yet any relevant information. Skipped.
		//DWARNING(("DipTypeClient::infoHandler got %s which is an empty string for type service %s\n", dimString, dimServiceName.getString()));
		LOG4CPLUS_WARN(DipLogger::getLoggerFromTopic(LOG_DIP_SUB_PREFIX,dimServiceName),LOG4CPLUS_TEXT(" Type service got an empty string from type service "<< dimServiceName.getString()));
		return;
	} else {
		const std::string rawDipData = DipTypeUtil::extractDipDataFromDimTypeServiceData(dimString);
		if (this->getQuality() == DIP_QUALITY_UNINITIALIZED){
			DipDimData dipData;
			DipLong time_ns = ((DipLong)getTimestamp() * (DipLong)1000000000L) + ((DipLong)getTimestampMillisecs());
			DipTimestamp ts;
			ts.setNanos(time_ns);
			dipData.setDataQuality(DIP_QUALITY_UNINITIALIZED);
			dipData.setQualityString(rawDipData);
			dipData.setDipTime(ts);
			dataObjectListener.dipDataReceived(dipData);
		}else{
			// Type service quality is good - so we have the data service format available
			dataFormatListener.gotDipDataFormat(rawDipData);
		}
	}
}
