#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include "Field.h"
#include "DipDimData.h"
#include "DipException.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>


Field::Field(const std::string &theFieldName, const DipDataType theFieldType, const unsigned int noElements):
holderForStringType(NULL),
data(NULL),
fieldName(theFieldName),
fieldType(theFieldType),
cardinality(noElements),
byteSize(0),
holderForCstrArray(NULL)
{
 
}



const std::string * Field::convertCstringToStandardStringType(){
	if (data == NULL){
		throw DipException(std::string("NO DATA"));
	}
	if (holderForStringType == NULL){
		if (fieldType == TYPE_STRING){
			holderForStringType = new std::string[1];
			holderForStringType[0] = (char *)data;
		} else if ((fieldType == TYPE_STRING_ARRAY) || (fieldType == TYPE_STRING_ARRAY_VAR)){
			int nstrs = cardinality;
			if(fieldType == TYPE_STRING_ARRAY_VAR)
				nstrs = *(int *)data;
			assert(holderForCstrArray != NULL);
			holderForStringType = new std::string[nstrs];
			for (unsigned int i = 0; i < nstrs; i++){
				holderForStringType[i] = holderForCstrArray[i];
			}
		} else {
			throw DipException(std::string("Field not a string type"));
		}
	}

	return holderForStringType;
}







Field::~Field(){
	eraseData();
}




void Field::eraseData(){
	if (holderForStringType){
		delete [] holderForStringType;
		holderForStringType = 0;
	}
	if (holderForCstrArray != NULL){
		delete [] holderForCstrArray;
		holderForCstrArray = NULL;
	}
	if (data != NULL){
		delete [](char *)data;
		data = NULL;
	}
}


void Field::setData(const void *theData, int nitems){
	if(fieldType >= TYPE_ARRAY_VAR_START)
	{
		cardinality = nitems;
	}
	setData(theData);
}


void Field::setData(const void *theData){
	if (data != NULL){
		eraseData();
	}
	char * clone = NULL;
	char *clonep, *strp;
	int i, nstrs;

	byteSize = 0;
	if((fieldType == TYPE_STRING_ARRAY) || (fieldType == TYPE_STRING_ARRAY_VAR)){
		nstrs = cardinality;
		for (i = 0; i < cardinality; i++){
			strp = ((char **)theData)[i];
			byteSize += strlen(strp) + 1;
		}
	} else if (fieldType == TYPE_STRING){
		strp = (char *) theData;
		byteSize = strlen(strp) + 1;
	} else{
		byteSize = DipDimData::getTypeSize(fieldType) * cardinality;
	}
	if(fieldType >= TYPE_ARRAY_VAR_START)
	{
		byteSize += sizeof(int);
	}
	clone = new char [byteSize];
	int size = byteSize;
	clonep = clone;
	if(fieldType >= TYPE_ARRAY_VAR_START)
	{
		memcpy(clonep, &cardinality, sizeof(int));
		clonep += sizeof(int);
		size -= sizeof(int);
		cardinality = 1;
	}
	if((fieldType == TYPE_STRING_ARRAY) || (fieldType == TYPE_STRING_ARRAY_VAR)){
		if(nstrs)
			holderForCstrArray = new const char *[nstrs];
		int len;
		for (i = 0; i < nstrs; i++){
			holderForCstrArray[i] = clonep;
			strp = ((char **)theData)[i];
			len = strlen(strp) + 1;
			memcpy(clonep, strp, len);
			clonep += len;
		}
	}else{
		memcpy(clonep, theData, size);
	}

	data = clone;
}





	
bool Field::equals(const Field &other) const{
	if ((cardinality != other.cardinality) || 
		(fieldName != other.fieldName)     ||
		(fieldType != other.fieldType)     ||
		(memcmp(data, other.data, byteSize) != 0)){
		return false;
	}
							
	return true;
}
