#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include "Service_DIPDataMapper.h"
#include "DipDimData.h"
#include "assert.h"
#include <stdio.h>
#include <stdlib.h>
#include "DipLogger.h"


void Service_DIPDataMapper::splitFormatStringIntoFields(std::string &formatString, std::vector<std::string> & stringFields){
	unsigned int startOfCurrentField = 0;

	while (startOfCurrentField < formatString.size()){
		unsigned int nextSeparator = formatString.find('|', startOfCurrentField);
		if (nextSeparator == std::string::npos || nextSeparator > formatString.size()){
			nextSeparator = formatString.size(); // just past eof string
		}

		unsigned int numCharsToCopy = nextSeparator - startOfCurrentField;
		numCharsToCopy = (numCharsToCopy < 0) ? 0 : numCharsToCopy; // This may happen if we encounter 2 seperators in a row


		try {
			std::string field = formatString.substr(startOfCurrentField, numCharsToCopy);
			stringFields.push_back(field);
		} catch (std::out_of_range & ex) {
			LOG4CPLUS_ERROR(log4cplus::Logger::getInstance("dip.system.dipdataservice"),LOG4CPLUS_TEXT("Error occured in splitFormatStringIntoFields : "<< ex.what()));
		} catch (...) {
			LOG4CPLUS_ERROR(log4cplus::Logger::getInstance("dip.system.dipdataservice"),LOG4CPLUS_TEXT("Error occured in splitFormatStringIntoFields while cutting away DIP Version info."));
		}
		/*
		Move over to the start of the next field - if nextSeperator = formatString.size() then we
		will exit.
		*/
		startOfCurrentField = nextSeparator +1;
	}
}

// CG	
void Service_DIPDataMapper::clearDipServiceMap(){// throws DipException
	dipDataFormatDescription = "";
	for (unsigned int i = 0; i < dipDataServiceFields.size(); i++){
		delete dipDataServiceFields[i];
	}
	dipDataServiceFields.clear();
}

void Service_DIPDataMapper::buildDipServiceMap(){// throws DipException
	if (!isFormatDefined()){
		throw DipException(std::string("dipDataFormatDescription is not defined"));
	}

	std::string structureDescription = dipDataFormatDescription;
		
	std::vector<std::string> structureParts;
	splitFormatStringIntoFields(structureDescription, structureParts);
	if ((structureParts.size() % 4) != 0){
		throw DipException("Bad structure descriptor("+structureDescription+") - Wrong number of fields (size % 4 != 0)");
	}
	bool foundVarType = false;
	int numBytesInFixedSection = 0;
	for (unsigned int i = 0; i < (structureParts.size() / 4); i++){
		std::string fieldName = structureParts[i * 4];
		int offSet = atoi(structureParts[(i * 4) + 1].c_str());
		int numBytes = atoi(structureParts[(i * 4) + 2].c_str());
		DipDataType type = (DipDataType)atoi(structureParts[(i * 4) + 3].c_str());
		if ((type == TYPE_STRING) || (type == TYPE_STRING_ARRAY) || (type >=  TYPE_ARRAY_VAR_START)){
			foundVarType = true;
		}else{
			// Not a string based field
			if (foundVarType){
				throw DipException("Bad structure descriptor("+structureDescription+") - field " + fieldName + "is not string type, yet found after fixed part description");
			}
			numBytesInFixedSection += numBytes;
			if ((offSet + numBytes) != numBytesInFixedSection){
				// Offsets should be examined in sequence - this will pick up any problems
				throw DipException(std::string("Bad structure descriptor") +structureDescription+" - field " + fieldName+" offSet+numBytes != totalNoBytes Processed so far - offsets out of sequence?");
			}
		}
		int cardinality = numBytes;
		if ((type != TYPE_STRING) && (type != TYPE_STRING_ARRAY) && (type <  TYPE_ARRAY_VAR_START)){
			cardinality = numBytes / DipDimData::getTypeSize((DipDataType)type);
			if ((cardinality < 1)||
				((type < 10) && (cardinality != 1))){
				throw DipException("Bad structure descriptor "+structureDescription+" - field " + fieldName + " has wrong cardinality");
			}
		}
	
			
		dipDataServiceFields.push_back(new DimServiceField(new Field(fieldName, type, cardinality), offSet));
	}
	startOfVarSection = numBytesInFixedSection;
}
	
/*
* This function is actally not usefull until DIP is required to be functionning on systems doing byte swapping.
* Then it shall not be protected anymore by the preprocessor directive and the variables "linux" and "WIN32".
*/
void Service_DIPDataMapper::buildDimFormatString(){
	bool foundStrings = false;
	for (unsigned int i = 0; i < dipDataServiceFields.size(); i++){
		Field & field = dipDataServiceFields[i]->dipField;
		DipDataType type = field.fieldType;
		if (type > 9){
			type = (DipDataType)((int)type/10);
		}

		char symbol;
		switch (type){
			case TYPE_BOOLEAN: 
			case TYPE_BYTE:
			case TYPE_STRING:	symbol = 'C'; break;
			case TYPE_SHORT:	symbol = 'S'; break;
			case TYPE_INT:		symbol = 'I';break;
			case TYPE_LONG:		symbol = 'X';break;
			case TYPE_FLOAT:	symbol = 'F';break;
			case TYPE_DOUBLE:	symbol = 'D';break;
			default: throw DipException(std::string("Excountered unknown DIP type while build DIM format descriptor"));
		}
		char dimFieldFormat[20];
		if (type == TYPE_STRING){
			sprintf(dimFieldFormat, "%c", symbol);
			foundStrings = true;
		}else{
			sprintf(dimFieldFormat, "%c:%d;", symbol, field.cardinality);
		}

		dimFormatDescription += dimFieldFormat;
		if (foundStrings){
			// string fields are always the last fields in the dipDataServiceFields list
			// they take up the rest of the space
			break;
		}
	}
	if (foundStrings == false){
		assert(dimFormatDescription[dimFormatDescription.size() - 1] != 'C');
		dimFormatDescription += "C";
	}
}
	

	


	
	
/**
 * Returns clone of field names
 * */
std::string* Service_DIPDataMapper::getFieldNames(int & noFields){// throws DipException
	if (dipDataServiceFields.size() == 0){
		buildDipServiceMap();
	}
		
	noFields = dipDataServiceFields.size();
	std::string * names = new std::string[noFields];
	for (int i = 0; i < noFields; i++){
		names[i] = dipDataServiceFields[i]->dipField.fieldName;
	} 
	return names;
}
	
	
	



	
Field & Service_DIPDataMapper::getField(const std::string & fieldName){ //throws DipException
	if (dipDataServiceFields.size() == 0){
		throw DipException(std::string("dipDataFormatDescription is not defined"));
	}
	for (unsigned int i=0; i < dipDataServiceFields.size(); i++){
		Field & field = dipDataServiceFields[i]->dipField;
		if (field.fieldName == fieldName){
			return field;
		}
	}
	throw DipException(std::string("Field ") + fieldName + " is not known in structure " + dipDataFormatDescription);
}
	
	
