#ifndef DipDataExtractor_h
#define DipDataExtractor_h

#include "Options.h"
#include "DipException.h"
#include "Service_DIPDataMapper.h"
#include "DataBlock.h"
#include "DipDimData.h"
#include "dic.hxx"



/**
 * @author beharrel
 *
 * Extracts data from a DIM service and inserts it into a DIPData object. According to the sequence defined by
 * Service_DIPDataMapper.
 * After this it extacts the ReasonForQuality string (an attribute of DIP data) and assigns this to the data object 
 * @see Service_DIPDataMapper
 */
class DipDataExtractor:public Service_DIPDataMapper {
public:
	void defineDipDataFormat(const std::string &format);// throws DipException
	void setDipDataFormatValidity(int valid);// throws DipException
	
	
	/**
	 * Extracts the data payload from the service and places it in the DIPData object. Note that the
	 * Reason for quality string is also extracted.
	 * This function is the exact inverse of writeDipObjectIntoDimService()
	 * @see DipDataInserter.writeDipObjectIntoDimService
	 * @return DipData - owned by caller
	 * */
	DipDimData * extractDipData(const void * data, const unsigned int byteSize) const;
	

	/**
	 * Used to extract both the Dip data and the Dip attributes 
	 * @return DipData - owned by caller
	 * */
	DipDimData*  extractDipDataObject(const void * data, unsigned int dataByteSize, DipQuality qual, longlong time_ns) const;
};
#endif

