#ifndef DipDataInserter_h
#define DipDataInserter_h

//#pragma warning(disable:4786)
#include "Options.h"
#include "Service_DIPDataMapper.h"
#include "DataBlock.h"
#include "DipDimData.h"
#include "SString.h"


/**
 * @author beharrel
 *
 * Writes data from the DIPData object into the DimService according to the sequnce defined in the 
 * super class Service_DIPDataMapper. To the end of this structure the reasonforquality(DIP data attribute is appended).
 * @see Service_DIPDataMapper
 */
class DipDataInserter :public Service_DIPDataMapper {
private:
	/// Holds the most recent data to be sent to the publication.- OWNED
	DataBlock * data;

	/// Set if the serialisation to the Dim service is to be verified
	static bool verifySerialization;

	
	/**
	 * Creates a structure description string based on the DipData object
	 * All primitives and arrays are entered at the start of the string (this describes the fixed part of the DIM service)
	 * followed by the string and array of strings (describing the variable part of the DIM service)
	 * */
	void setDipStructureString(const DipDimData & data);

public:
	static const SString emptyQualityReasonMessage;
	
	DipDataInserter():data(NULL){};
	

	virtual ~DipDataInserter(){
		if (data){
			delete data;
		}
	}




	/**
	 * Writes the fixed and varaible parts of the structure, finally writes the quality reason dip attribute (which is a null string
	 * since data has been sent - so data must be good quality). 
	 * 
	 * Only if Everything is ok do we update the service info.
	 * */
	void writeDipObjectIntoDataBlock(const DipDimData & dipData);// throws DipException, TypeMismatch {



	void setQualityExplainationString(const std::string &reason);




	const void * getDataBlock(unsigned int &blockSize){
		blockSize = data->getNumBytesStored();
		return data->read(blockSize, 0);
	}
};


#endif
