#ifndef DIPDATASERVICE
#define DIPDATASERVICE

#include "Options.h"
#include "SString.h"
#include "dis.hxx"
#include "DipQuality.h"
#include "DipTimestamp.h"
#include "DipData.h"

class DipDataInserter;


// This sevice provides the client the data for the publication they are subscribed to.
class DipDataService{

private:
	DipQuality dataQuality;

	DipLong time_ns;


	SString reasonForQuality;


	// We do not know the type of the service until data it transmitted - so we must defer the creation of the DiM data service untill data it first sent. 
	DimService *dimService;


	/// Used to serialize a DipData object and place it in the DIM data service
	DipDataInserter * dipDataSerializer;


	void sendDipData();

	void sendQualityIfChanged(DipQuality quality, const SString &reason);
public:
	/**
	* @parameter dataSerializer - ownership passed to this object
	*/
	DipDataService(const std::string & dipName, DipDataInserter * dataSerializer, const DipData & data, const DipQuality dipQuality, const SString qualityReason );

	~DipDataService();

	void setQualityUncertain(const SString &reason);
	void setQualityBad(const SString &reason);
	void resendToClient(int clientID);

	void updateDipData(const DipData & data);
	void updateDipData(const DipData & data, const DipQuality quality, const SString qualityReason);

	static const std::string dipDataServiceEnding;
};

#endif

