#ifndef DipDimBrowser_h
#define DipDimBrowser_h

#include "Options.h"
#include "DipBrowser.h"
#include "DipDataExtractor.h"
#include "dic.hxx"



class DipDimBrowser:public DipBrowser {
private:
	DipDataExtractor *publicationDataExtractor;
	
	DimBrowser dimNameSpaceBrowser;


	/**
	* Holds list of publication names.
	* Populated by getPublications()
	*/
	char ** pubNames;

	/**
	* number of publications retrieved by last getPublications() call.
	*/
	int numberOfPublications;



	/**
	* Holds list of field names for a publication.
	* Populated by getTags(). Actual values held in fieldNamesAsStringArray
	*/
	char ** fieldNames;



	/**
	* holder for the string array returned by publicationDataExtractor.defineDipDataFormat.
	* Populated by getTags(),
	*/
	std::string * fieldNamesAsStringArray;

	/**
	* number of field names retrieved by last getTags() call.
	*/
	int numberOfFields;



	void destroyPubList(){
		if (pubNames){
			for (int i = 0; i < numberOfPublications; i++){
				delete [] pubNames[i];
			}
			delete [] pubNames;
			pubNames = NULL;
			numberOfPublications = 0;
		}
	}


	void destroyFieldList(){
		if (fieldNamesAsStringArray){
			delete [] fieldNames;
			fieldNames = NULL;
		
			delete [] fieldNamesAsStringArray;
			fieldNamesAsStringArray = NULL;
		}
			
		numberOfFields = 0;
	}

public:

	DipDimBrowser(): publicationDataExtractor(NULL), 
		pubNames(NULL),numberOfPublications(0), fieldNames(NULL), 
		fieldNamesAsStringArray(NULL), numberOfFields(0)
	{
	} 


	virtual ~DipDimBrowser(){

		if (publicationDataExtractor){
			delete publicationDataExtractor;
		}

		destroyPubList();
		destroyFieldList();
	}
	

	const char ** getPublications(const char * pattern, unsigned int &noPublications);
	

	
	const char ** getTags(const char * pub, unsigned int &noFields);
	
	/**
	 * Gets the type of the last publication queried by getTags
	 * Used where publication is primitive 
	 * @see getTags
	 * */
	int getType()
	{
		return getType(DipDimData::fieldNameWhenPrimitive);
	}
	
	/**
	* Gets the cardinality of the last publication queried by getTags
	* Used where publication is primitive 
	* @see getTags
	* */
	int getSize()
	{
		return getSize(DipDimData::fieldNameWhenPrimitive);
	}
	
	/**
	 * Gets the type of a tag in the last publication queried by getTags
	 * @see getTags
	 * */
	int getType(const char * tag);
	


	/**
	* Gets the cardinality of a tag in the last publication queried by getTags
	* @see getTags
	* */
	int getSize(const char * tag);
};




#endif
