#ifndef DipDimClient_h
#define DipDimClient_h

#include "Options.h"
#include <string>
#include "dic.hxx"
#include "DipSubscriptionListener.h"
#include "DipSubscription.h"

#include "DipTypeClient.h"
#include "DipDataClient.h"


class DipDimClient:public DipDataObjectListener, public DipDataFormatListener, public DisconnectListener
{
private:
	/// Possible states we expect the publisher to be in (from the client perspective)
	enum DIPPUBSTATES{Uninit, PubDoesNotExist, PubConnected, PubStructDef, PubDisconnected, PubRestarting, PubStructReDef};

	char const* dipPuStatesStrings[20];
	/// Keep track of the publisher state from the clients perspective
	DIPPUBSTATES dipPublisherState;


	/**
	* Dim service that gives us the DIP data structure
	*/ 
	DipTypeClient * dipTypeService;


	/**
	* Dim service that gives us the DIP data
	*/ 
	DipDataClient *dipDataService;


	/**
	* Ref to our call back to handle DIP events on the client side
	* May be used by many subscriptions!
	*/
	DipSubscriptionListener &dimListener;


	/**
	 * the Subscription this client is subscribed to
	 * */
	DipSubscription &dimSubscription;

	/**
	 * Last known quality - used for logging purposes only.
	 */
	DipQuality lastKnownQuality;

	/**
	* Name of publication
	*/
	std::string pubName;


	void signalConnectionToListener();
	void signalDisconnectToListener();
	void signalDipDataRxToListener(const DipData &data);
	void signalAsynchErrorToListener(DipException & de);
				
public:

	DipDimClient(const std::string & theTopicName, DipSubscriptionListener & listener, DipSubscription & subscription);


	virtual ~DipDimClient();

	/**
	* Request a data resend from the publisher
	*/
	void doRequest()
	{
		DimClient::sendCommandNB((pubName+"/DipUpdate").c_str(),1);
	}

	
	/**
	* Callback for data - either from the DipTypeService (when handling 
	* reason for qulity strings) or the DipDataService
	*/
	void dipDataReceived(const DipData & data);

	/**
	* Used by DipTypeService to indicate a format string has been received
	*/
	void gotDipDataFormat(const std::string & format);

	/**
	* Used by DipTypeService to indicate we have lost touch with the type service,
	* And thus the whole publication.
	*/
	void disconnected(const DimStampedInfo & source);
};

#endif
