// DipDimFactory.h: interface for the DipDimFactory class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIPDIMFACTORY_H__1294F7FE_D1C3_4FF5_8977_E2881A5DE365__INCLUDED_)
#define AFX_DIPDIMFACTORY_H__1294F7FE_D1C3_4FF5_8977_E2881A5DE365__INCLUDED_

#ifdef WIN32
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#pragma warning(disable:4786)
#endif

#include "Options.h"
#include "DipFactory.h"
#include "DipDimSubscription.h"
#include "platformDependant.h"
#include <map>
#include <iostream>
#include "log4cplus/configurator.h"
#include "dis.hxx"

using namespace std;

struct CStrLess  {
	bool operator()(const char * _X, const char * _Y) const
		{return (strcmp(_X,_Y) < 0); 
		}
	};
/*
class CommunicationErrorHandler: public DimErrorHandler {

	virtual void errorHandler (int severity, int errorCode, char *errorMsg ) {
		string exceptionText = errorMsg;
		throw DipException(exceptionText);
		}
	};
*/

class DipDimFactory : public DipFactory {
private:
	DipDimFactory(const DipDimFactory& other);				// Disallow copying
	DipDimFactory& operator=(const DipDimFactory& other);	// ...and assignment

	/*
	* To catch DIM error rather than they go straight to the stdout, therefore useless...
	*/
	/*CommunicationErrorHandler * errorHandler;*/

	/**
	* name of the server
	*/
	std::string dimServerName;

	/**Here are declared the default values for the DipFactory to reach the global CERN Public Network DNSs*/
	static const char* defaultDnsList;//="DIPNSDEV.cern.ch";

	/**The default port cannot be overriden*/
	static const int defaultDnsPort;// = 2506;

	/**The default environment variable cannot be overriden*/
	static const char* defaultDnsEnvVarName;// = "DIPNS";

	/** Indicates whether the DIM server has been already started or not.*/
	bool dimServerStarted;

	/**
	* Serves to serialise publication creation and destruction
	*/
	SMutex lock;

	/**
	* DIP does not allow publishers to duplication publications
	* this will prevent duplicates on this host. DIM will handle things
	* between hosts.
	* No fields owned by publist.
	*/
	std::map<const char *, const DipPublication *, CStrLess> pubList;

//	boost::property_tree::ptree m_diagnosticPTree;
	log4cplus::ConfigureAndWatchThread* log4cplusConfigureAndWatchThread;

	DimService *dipVersionService;

public:

	DipDimFactory(const char *username = NULL, const char* logPropertiesFile = "log4cplus.properties", int loggingConfigWatchInterval = -1);

	~DipDimFactory();

	void setName(const char *username);

	void setDNSNode(const char* nodename);

	void setTimeout(const int timeout);
	int getTimeout();

	DipSubscription* createDipSubscription(const char *topicname, DipSubscriptionListener* listener);

	DipPublication* createDipPublication(const char *topicname, DipPublicationErrorHandler* listener);

	void destroyDipSubscription(DipSubscription* subscription);

	void destroyDipPublication(DipPublication* publication);

	DipData* createDipData();

	DipBrowser* createDipBrowser();

	/**
	* This must be used to update the DIM DNS when a service or command is added.
	*/
	//void registerWithDimNS();
	};

#endif // !defined(AFX_DIPDIMFACTORY_H__1294F7FE_D1C3_4FF5_8977_E2881A5DE365__INCLUDED_)
