#ifndef DipDimPublication_h
#define DipDimPublication_h

#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include "Options.h"
#include "DipPublication.h"
#include "DipDimServer.h"
#include "DipPublicationErrorHandler.h"


/**
 * Interface for DIP publications. DipPublication objects are created by DipFactoryInterface
 * objects.
 * Dip Publications can send simple data types or DipData objects. All send methods take as
 * arguments the value to be sent, the timestamp which will accompany the the value and the
 * quality information.
 */
class DipDimPublication:public DipPublication
{
private:
	std::string theName;
	DipDimServer *dimServer;
	DipPublicationErrorHandler &handler;

public:
	DipDimPublication(std::string &topicname, DipPublicationErrorHandler &theHandler);

	~DipDimPublication(){
		if (dimServer){
			delete dimServer;
		}
	}


	const char * getTopicName() const
	{
		return theName.c_str();
	}



		
	void send(const char *value,	const DipTimestamp & timestamp);
	void send(const std::string &value,	const DipTimestamp & timestamp);
	void send(DipBool value,		const DipTimestamp & timestamp);
	void send(DipByte value,		const DipTimestamp & timestamp);
	void send(DipShort value,		const DipTimestamp & timestamp);
	void send(DipInt value, 		const DipTimestamp & timestamp);
	void send(DipLong value, 		const DipTimestamp & timestamp);
	void send(DipFloat value, 		const DipTimestamp & timestamp);
	void send(DipDouble value, 		const DipTimestamp & timestamp);
	void send(const DipData &value, const DipTimestamp & timestamp);
    void send(const DipData &value, const DipTimestamp & timestamp, DipQuality quality, const char * qualityReason);
	void send(const DipBool *value, int size, const DipTimestamp & timestamp);
	void send(const DipByte *value, int size, const DipTimestamp & timestamp);
	void send(const DipShort*value,	int size, const DipTimestamp & timestamp);
	void send(const DipInt *value, 	int size, const DipTimestamp & timestamp);
	void send(const DipLong *value, int size, const DipTimestamp & timestamp);
	void send(const DipFloat *value,int size, const DipTimestamp & timestamp);
	void send(const DipDouble*value,int size, const DipTimestamp & timestamp);
	void send(const std::string*value,int size,const DipTimestamp & timestamp);
	void send(const char **value,	int size, const DipTimestamp & timestamp);
	
  
  
	void setQualityBad()
	{
		setQualityBad(DipDataInserter::emptyQualityReasonMessage);
	}
  
	void setQualityBad(const char * reason);
  
  
	void setQualityUncertain()
	{
		setQualityUncertain(DipDataInserter::emptyQualityReasonMessage);
	}
  
	void setQualityUncertain(const char *reason);
};


#endif
