#ifndef DipDimServer_h
#define DipDimServer_h

#include "Options.h"
#include "dis.hxx"
#include "DipDataInserter.h"
#include "DipTimestamp.h"
#include "DipPublication.h"

#include "DipTypeService.h"
#include "DipDataService.h"
#include <iostream>

using namespace std;



class DipDimFactory;


class DipDimServer
{
private:
	class DipUpdateCommand:public DimCommand{
	private:
		DipDimServer & publisher;
	public:
		DipUpdateCommand(const std::string & name, DipDimServer & pub):DimCommand(name.c_str(), "I"),
			publisher(pub){
		};

		// delegates to the publisher - who resends the last published data to the requester only.
		void commandHandler(){
			publisher.updateSingleClient();
		}
	};

	friend class DipDimServer::DipUpdateCommand;


	std::string dimName;
	
	///Publishes data structure of DIP data in DIM when it is known. Whilst the data strucure is unknown this service is 
	///used to transmit the reasonForQuality string
	DipTypeService *dimTypeService;
	
	/// Publishes the Dip data in DIM
	DipDataService *dimDataService;
	
	/// Handles the update request from a client
	DipUpdateCommand *dimCommand;


public:
	DipDimServer(const std::string & topicname, DipPublication & publication);
	

	~DipDimServer(){
		if (dimDataService != NULL){
			delete dimDataService;
		}

		if (dimCommand != NULL){
			delete dimCommand;
		}

		if (dimTypeService != NULL){
			delete dimTypeService;
		}
	}

	
	
	void send(const DipDimData & value);
	
	void send(const DipDimData & value, const DipQuality quality,const SString qualityReason);
	

	/**
	 * If the data service is defined (data has been sent at least once through the publication) then
	 * quality information is sent through the data service - otherwise it is sent through the type service
	 * @see dimDataService
	 * @see dimTypeService
	 * */
	void resendWithNewQuality(DipQuality quality, const SString & reason);
	
	
	
	/**
	 * Resends last sent data to the client that requested it.
	 * */
	void updateSingleClient();
};
#endif
