#ifndef DIPLOGGER_H
#define DIPLOGGER_H

#include "log4cplus/logger.h"
#include "log4cplus/loglevel.h"
#include "log4cplus/loggingmacros.h"
#include "log4cplus/configurator.h"
#include <iostream>
#include <fstream>
#include <string.h>

#define LOG_DIP_PUB_PREFIX ( "dip.publication.")
#define LOG_DIP_SUB_PREFIX ( "dip.subscription.")

using namespace std;
using namespace log4cplus;

class DipLogger {
public:
	static void findAndReplace( std::string& source, const char* find, const char* replace )
	{
	   size_t findLn = strlen(find);
	   size_t replaceLn = strlen(replace);
	   size_t pos = 0;

	   //search for the next occurrence of find within source
	   while ((pos = source.find( find, pos)) != std::string::npos)
	   {
		  //replace the found string with the replacement
		  source.replace( pos, findLn, replace );

		  //the next line keeps you from searching your replace string, 
		  //so your could replace "hello" with "hello world" 
		  //and not have it cause problems.
		  pos += replaceLn; 
	   }
	}


	static std::string getDotSeparatedTopicName(std::string prefix, std::string topic, std::string suffix=""){
		std::string result(prefix);
		result.append(topic);

		// Remove all characters that logging library cannot handle
		findAndReplace(result, "/", ".");
		findAndReplace(result, ":", ".");
		findAndReplace(result, "=", ".");
		findAndReplace(result, "[", "");
		findAndReplace(result, "]", "");

		return result;
	}


	static log4cplus::Logger getLoggerFromTopic(std::string prefix, std::string topic, std::string suffix=""){
		return log4cplus::Logger::getInstance(LOG4CPLUS_TEXT(getDotSeparatedTopicName(prefix, topic, suffix)));
	}

	static log4cplus::ConfigureAndWatchThread* initLogging(std::string filename = "log4cplus.properties", int loggingConfigurationWatchInterval = -1){
		    Logger root = Logger::getRoot();
			ConfigureAndWatchThread* configureThread = NULL;
		    try {
				if (loggingConfigurationWatchInterval == -1){
					std::ifstream fi(filename.c_str());
					if(fi.good()){
					  fi.close();
					  log4cplus::PropertyConfigurator::doConfigure(LOG4CPLUS_TEXT(filename));
					  LOG4CPLUS_INFO(root,LOG4CPLUS_TEXT("Configured Log4cplus from properties file"));
					}else{
					  cout << LOG4CPLUS_TEXT("For Information : DIP API Logging not configured, cannot access '"<<filename<<"'.") << endl;

					}
				}else{
					if (loggingConfigurationWatchInterval < 500){
					  cout << LOG4CPLUS_TEXT("Logging configuration watching interval cannot be less than 500 ms."<<endl<<"Now stopping DIP.") << endl;
					  exit(-1);
					}else{
						std::ifstream fi(filename.c_str());
						if(fi.good()){
							fi.close();
							configureThread = new ConfigureAndWatchThread(
								LOG4CPLUS_TEXT(filename), loggingConfigurationWatchInterval);
							LOG4CPLUS_INFO(root,LOG4CPLUS_TEXT("Configured Log4cplus config watch interval to "<<loggingConfigurationWatchInterval<<" ms."));
						}else{
						  cout << LOG4CPLUS_TEXT("For Information : DIP API Logging not configured, cannot access '"<<filename<<"'.") << endl;
						}
					}
				}
			}
			catch(...) {
				cout << LOG4CPLUS_TEXT("Exception while configuring log4cplus...") << endl;
			    LOG4CPLUS_FATAL(root,LOG4CPLUS_TEXT("Exception while configuring log4cplus..."));
			}
			return configureThread;

	}

};

#endif
