#ifndef DIPTYPEUTIL
#define DIPTYPEUTIL

#include "DIPVersion.h"

#include "DipLogger.h"

/**
* The DIM type service carries the DIP version string (9 chars) + DIP data (maybe quality 
* string or type info depending on the publisher state). This utility class is used for
* creating and extracting this data.
*/
class DipTypeUtil{
public:
	/**
	* Build data ready for sending to the Dim Type service
	*/
	static std::string buildDimTypeServiceData(const std::string & dipData){
//		assert(DipVersion::getDipVersionStringLen() == DipVersion::getDipVersion().size());
		return (DipVersion::getPublicationDipVersion() + dipData);
	}

	/**
	* Extract DIP data from the data received from the Dim type service
	*/
	static std::string extractDipDataFromDimTypeServiceData(const std::string & dimData){
		std::string result;
		//No MSVC 6.0 supported: result.clear();
		try {
			result = dimData.substr(DipVersion::getDipVersionStringLen(dimData), dimData.size() - DipVersion::getDipVersionStringLen());
			} catch (std::out_of_range & ex) {
//				DERROR(("Error occured in extractDipDataFromDimTypeServiceData : %s\n", ex.what()));
				LOG4CPLUS_ERROR(log4cplus::Logger::getInstance("dip.system.diptypeutil"),LOG4CPLUS_TEXT("Error occurred in extractDipDataFromDimTypeServiceData : "<< ex.what()));
			} catch (...) {
//				DERROR(("%s\n", "Error occured in extractDipDataFromDimTypeServiceData while cutting away DIP Version info."));
				LOG4CPLUS_ERROR(log4cplus::Logger::getInstance("dip.system.diptypeutil"),LOG4CPLUS_TEXT("Error occured in extractDipDataFromDimTypeServiceData while cutting away DIP Version info."));
			}
		return (result); 
	}


	/**
	* Extract the DIP version string from data received from the Dim type service
	*/
	static std::string extractDipVersionFromDimTypeServiceData(const std::string & dimData){
		return (dimData.substr(0,(DipVersion::getDipVersionStringLen(dimData)-1)));
	}
};

#endif
