#ifndef Field_h
#define Field_h
#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include "Options.h"
#include <string>
#include "DipDataType.h"

/**
 * @author beharrel
 * Holds information about fields of a DIP publication.
 */
class Field {
private:
	/**
	* For consitent data retreival in DipDimData, may be used to hold primitive
	* or array of std::string
	*/
	std::string * holderForStringType;

public:
	/// The data held in the field
	void * data;
	/// The field name
	const std::string fieldName;
	/// the field type 
	const DipDataType fieldType;
	/// tells us how many elements there are
	unsigned int cardinality;
	/// tells us how many bytes are being used to hold the data
	unsigned int byteSize;
	/**
	* holds ptrs to c strings if the field is a array of strings
	* Using this allows to consistent treatment of arrays 
	* in DipDimData (so caller does not have to delete retruned array)
	*/
	const char** holderForCstrArray;

		
	/**
	* The String types are passes as cstrings. 
	* string arrays are stored in a single block of memory
	* @param theData is cloned
	* */
	Field(const std::string &theFieldName, const DipDataType theFieldType, const unsigned int noElements);
	

	~Field();


	bool equals(const Field &other) const;

	const std::string * convertCstringToStandardStringType();

	/// delete data held by the field
	void eraseData();

	/// must be of the same type as the original - will delete old data it exists
	void setData(const void *theData);
	void setData(const void *theData, int cardinality);
};

#endif

