#include <gtest/gtest.h>

#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

#include <stdio.h>
#include <string>
#include <iostream>
#include <sstream>
#include <pthread.h>
#include "gtest_main.hpp"

#include "Dip.h"
#include "DipSubscription.h"


using namespace std;

string    someString = "test";
DipBool	  someBool   = true;
DipByte	  someByte   = 1;
DipShort  someShort  = 1;
DipInt	  someInt    = 1;
DipLong	  someLong   = 1073741824l;
DipFloat  someFloat  = 1.0f;
DipDouble someDouble = 1.0f;

int sizefields = 10;

string*    someStringArray = new string[sizefields];
DipBool*   someBoolArray   = new DipBool[sizefields];
DipByte*   someByteArray   = new DipByte[sizefields];
DipShort*  someShortArray  = new DipShort[sizefields];
DipInt*    someIntArray    = new DipInt[sizefields];
DipLong*   someLongArray   = new DipLong[sizefields];
DipFloat*  someFloatArray  = new DipFloat[sizefields];
DipDouble* someDoubleArray = new DipDouble[sizefields];

void initArrays(){
	for (int i = 0 ;i < sizefields ; i++) {
		someStringArray[i] = '\0';
		someBoolArray[i] = true;
		someByteArray[i] = 0;
		someShortArray[i] = 0;
		someIntArray[i] = 0;
		someLongArray[i] = 0;
		someFloatArray[i] = 0.0f;
		someDoubleArray[i] = 0.0f;
    }
}
		
		

class LocalErrHandler:public DipPublicationErrorHandler{
  public:
	void handleException(DipPublication* publication, DipException& ex){
		 cout << "DIP publication error:" << ex.what()<<endl;
	}
};

	
class LocalDataListener:public DipSubscriptionListener{
        
    public:
        int message_count;
        double double_total;
        int error_count;
        
        LocalDataListener(){
          message_count = 0;
          double_total = 0.0;
          error_count = 0;
        };
    
        void handleMessage(DipSubscription *subscription, DipData &message){
            if(message.contains("doubleValue")){
              message_count++;
              double_total += message.extractDouble("doubleValue");
            }
        }
    
        void connected(DipSubscription *subscription) {
            //cout << "\nPublication source  " << subscription->getTopicName()<< " available\n";
        }
    
        void disconnected(DipSubscription *subscription, char *reason) {
            //printf("\nPublication source %s unavailable : %s\n", subscription->getTopicName(), reason);
        }
    
        void handleException(DipSubscription* subscription, DipException& ex){
            printf("Subs %s has error %s\n", subscription->getTopicName(), ex.what());
            error_count++;
         }
    
    };
    
struct thread_data{
   DipFactory* dip;
   DipPublication** pubs;
   int numberOfPubs;
};    
    
void pushDipData(void* args){
    struct thread_data *threadArgs;
    threadArgs = (struct thread_data *) args;
    DipFactory* dip = threadArgs->dip;
    DipPublication** pubs = threadArgs->pubs;
    int numberOfPubs = threadArgs->numberOfPubs;
    
    for(int i = 0 ; i < numberOfPubs ; i++){
        DipTimestamp time;
        DipData* dat = dip->createDipData();
        dat->insert(someString,"stringValue");
        dat->insert(someDouble,"doubleValue");
        dat->insert(someInt,"intValue");
        dat->insert(someBool,"booleanValue");
        pubs[i]->send(*dat, time);
        delete dat;
   }
   
   pthread_exit(NULL);
}

    
TEST(sanityTests, simpleRoundtripTest) {
   DipSubscription **sub;
   DipPublication **pub;
   DipFactory *dip;
   LocalDataListener handler;
   LocalErrHandler errHandler;
   

   int numberOfPubs = 30;
   initArrays();
   
   dip = Dip::create("simpleRoundtripTest");
   dip->setDNSNode("dipnsdev");
   
   sub = new DipSubscription*[numberOfPubs];
   pub = new DipPublication*[numberOfPubs];
   
   cout << "Creating subscriptions and publications..."<<endl;
   for(int i = 0 ; i < numberOfPubs ; i++){
        ostringstream oss;
        oss << "dip/local/sanity/test/pub_";
        oss.fill('0');
        oss << setw(3) << i;
        sub[i] = dip->createDipSubscription(oss.str().c_str(), &handler);
        pub[i] = dip->createDipPublication(oss.str().c_str(), &errHandler);
   }
   cout << "Pushing publications data..."<<endl;
   
   pthread_t thread;
   void *threadStatus;
   struct thread_data td;
   td.dip = dip;
   td.pubs = pub;
   td.numberOfPubs = numberOfPubs;
   
   
   pthread_create(&thread, NULL, pushDipData, (void *) &td);
   cout << "Waiting for pushDipData thread... "<<endl;
   pthread_join(thread, &threadStatus);
   
   cout << "Sleeping"<<endl;
   sleep(2);
   cout << "Destroying subscriptions and publications.."<<endl;
    for(int i = 0 ; i < numberOfPubs ; i++){
        dip->destroyDipSubscription(sub[i]);
        dip->destroyDipPublication(pub[i]);
    }
    cout << "Sleeping"<<endl;
    sleep(1);
   
   cout << "Now shutting down."<<endl;
   
   Dip::shutdown();
   
   EXPECT_EQ(handler.message_count, numberOfPubs);
   EXPECT_EQ(handler.double_total, (double)numberOfPubs);
   EXPECT_EQ(handler.error_count, 0);
}
/*
TEST(ensTests, DISABLED_remusSubscriptionTest) {
   DipSubscription **sub;
   DipFactory *dip;
   LocalDataListener handler;

   int numberOfPubs = 64;
   
   dip = Dip::create("dip-basic-test-client");
   dip->setDNSNode("dipns1,dipns2");
   
   sub = new DipSubscription*[numberOfPubs];
   
   for(int cycle = 0 ; cycle < 5 ; cycle++){
        cout << "Creating subscriptions"<<endl;
       for(int i = 0 ; i < numberOfPubs ; i++){
            ostringstream oss;
            oss << "dip/REMUS/ERGO/BGSTAB";
            oss.fill('0');
            oss << setw(2) << i;
            sub[i] = dip->createDipSubscription(oss.str().c_str(), &handler);
       }
        cout << "Sleeping"<<endl;
        sleep(1);
        cout << "Destroying subscriptions"<<endl;
        for(int i = 0 ; i < numberOfPubs ; i++){
            dip->destroyDipSubscription(sub[i]);
        }
        cout << "Sleeping"<<endl;
        sleep(1);
   }
   
   
   sleep(1); 
   EXPECT_GT(handler.message_count, 1);
   EXPECT_GT(handler.double_total, 1.0);
   EXPECT_EQ(handler.error_count, 0);
   
   cout << "Now shutting down"<<endl;
   
   Dip::shutdown();
}
*/
