import groovy.util.XmlParser 
import groovy.util.XmlSlurper
import groovy.xml.MarkupBuilder
import cern.dip.*

dipTypeNameToInt = [
"NULL":0
,"BOOLEAN":1
,"BOOLEAN_ARRAY":10
,"BYTE":2
,"BYTE_ARRAY":20
,"SHORT":3
,"SHORT_ARRAY":30
,"INT":4
,"INT_ARRAY":40
,"LONG":5
,"LONG_ARRAY":50
,"FLOAT":6
,"FLOAT_ARRAY":60
,"DOUBLE":7
,"DOUBLE_ARRAY":70
,"STRING":8
,"STRING_ARRAY":80
]

xsdTypeNameToInt = [
"xsd:null":0
,"xsd:bool":1
,"BOOLEAN_ARRAY":10
//,"xsd:byte":2
//,"BYTE_ARRAY":20
//,"xsd:int":3
//,"SHORT_ARRAY":30
,"xsd:int":4
,"INT_ARRAY":40
,"xsd:long":5
,"LONG_ARRAY":50
,"xsd:float":6
,"FLOAT_ARRAY":60
,"xsd:string":8
,"STRING_ARRAY":80
]


dipTypeIntToName = dipTypeNameToInt.collectEntries { e -> [(e.value): e.key] }
dipTypeIntToXsdName = xsdTypeNameToInt.collectEntries { e -> [(e.value): e.key] }

writer = new StringWriter()
xml = new MarkupBuilder(writer) 


println "Collecting publications for "+this.args[0]

factory = Dip.create("fwDipImport-generator")
browser = factory.createDipBrowser()


publications = browser.getPublications(this.args[0])

xml.'configuration'(publisherName:"ccc7_23824",queryTimeout:"15",dns:"dipns1,dipns2",managerNumber:"1",name:"DIPConfig_1"){
  publications.each{ 
      def pub = it
      browser.getTags(it).each{ tagName ->
        def dipType = browser.getType(tagName)
        def xsdType = dipTypeIntToXsdName[dipType]?:dipType
        def dipTypeName = dipTypeIntToName[dipType]
        def dpeName = pub.replaceAll(/[\/\\-]/,"_") + "_"+ (tagName=="__DIP_DEFAULT__"?"default":tagName)
        'subscription'(type:"single",name:pub ,xsdType: xsdType){
          dpe{
            name ( dpeName )
            alias()
            element(".")
            tag (tagName)
          }
          overwrite ("TRUE")
          type("ExampleDP_"+dipTypeName.toLowerCase().capitalize())
          description(dpeName+".")
          unit()
        }
        
      }
  }
}
new File(this.args[1]) <<  writer.toString()

