// DIP.cpp : Defines the entry point for the DLL application.
//
//#include "Stdafx.h"
#include "DIPBridge.h"
#include "Dip.h"
#include "DIPVersion.h"
//#include "DIMVersion.h"
#include "dim.h"


DipFactory * DipJavaCPPBridge::factory = NULL;

#ifdef WIN32
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			printf("Running %s\n", DipVersion::getDipVersion().c_str());
			//printf("Using DIM %s\n", DIM_VERSION_NUMBER);
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#else
void _init(){
	printf("Running %s\n", DipVersion::getDipVersion().c_str());
	//printf("Using DIM %s\n", DIM_VERSION_NUMBER);
}
#endif


//Offered possibility to tell the Factory which DNS to use. The factory is already present by construction.
JNIEXPORT void JNICALL Java_cern_dip_implementation_DipFactoryImp__1setDNSNode (JNIEnv * env, jobject, jstring dnsName)
{
	const char * DNSName_cstr = env->GetStringUTFChars(dnsName, NULL);
	DTRACE(("A DNS has been specified via the DIP API.\n"));
	DipJavaCPPBridge::getFactory().setDNSNode(DNSName_cstr);
	if (NULL != DNSName_cstr) {
		env->ReleaseStringUTFChars(dnsName, DNSName_cstr);
	}
}


JNIEXPORT void JNICALL Java_cern_dip_implementation_DipFactoryImp__1setTimeout (JNIEnv *, jobject, jint timeout)
{
	DTRACE(("A DIP/DIM timeout has been specified via the DIP API : %d\n",timeout));
	DipJavaCPPBridge::getFactory().setTimeout(timeout);
}

JNIEXPORT jint JNICALL Java_cern_dip_implementation_DipFactoryImp__1getTimeout
(JNIEnv *, jobject)
{
	return DipJavaCPPBridge::getFactory().getTimeout();
}

JNIEXPORT void JNICALL Java_cern_dip_Dip__1setUpDIPServer
(JNIEnv * env, jclass, jstring serverName){
	const char * serverName_cstr = serverName ? env->GetStringUTFChars(serverName, NULL) : NULL;
#ifndef WIN32
  // Display the DIP version number o
  _init();
#endif
	// TODO check for exceptions
	DipFactory * factory;
	if (serverName_cstr == NULL){
		factory = Dip::create();
	} else{
		factory = Dip::create(serverName_cstr);
	}
	DipJavaCPPBridge::setFactory(factory);
	if (serverName_cstr){
		env->ReleaseStringUTFChars(serverName, serverName_cstr);
	}
}

JNIEXPORT jstring JNICALL Java_cern_dip_Dip__1getDIPVersion
  (JNIEnv *env, jclass) {
    return  env->NewStringUTF(DipVersion::getDipVersion().c_str());
}

/**
* note that this does NOT interrupt the flow of the program. It just flags an exception that
* will be thrown at the Java implementation when the C++ implementation has returned.
*/
void DipJavaCPPBridge::flagJavaDIPException(JNIEnv *env, const char * reason){
	// I should ony have to do this at init time - but this is causing an occasional crash hence I do
	// it every time the method is called - this seems to have solved the crashing problem - why?
	// I'm not so sure - maybe the timing has just changed!
	/*static*/ jclass jExceptionClass = env->FindClass("cern/dip/DipException"); 
	DTRACE(("In DipJavaCPPBridge::flagJavaDIPException\n"));
	DTRACE(("DipJavaCPPBridge::flagJavaDIPException - Exception str is %s\n", reason));
    env->ThrowNew(jExceptionClass, reason);	
	DTRACE(("OutDipJavaCPPBridge::flagJavaDIPException\n"));
}
