#include "DIPBridge.h"
#include "DipBrowser.h"

static CdipObjectPool<DipBrowser> browsePool("browserPool");

JNIEXPORT jint JNICALL Java_cern_dip_implementation_DipBrowserImp__1create
(JNIEnv *, jobject){
	DTRACE(("In DipBrowserImp__1create\n"));
	DipBrowser * browser = DipJavaCPPBridge::getFactory().createDipBrowser();
	DTRACE(("Leaving DipBrowserImp__1create\n"));
	return browsePool.give(browser);
}

JNIEXPORT void JNICALL Java_cern_dip_implementation_DipBrowserImp__1destroy
(JNIEnv *, jobject, jint objectIndex){
	DTRACE(("In DipBrowserImp__1destroy\n"));
	DipBrowser * browser = browsePool.remove(objectIndex);
	delete browser;
	DTRACE(("Out DipBrowserImp__1destroy\n"));
}

jobjectArray makeJavaStringArray(JNIEnv *env, const char ** c_strs, unsigned int noStrings){
	DTRACE(("In makeJavaStringArray\n"));
	jclass stringArrayClass = env->FindClass("java/lang/String");
	jobjectArray jStrings = (jobjectArray)env->NewObjectArray(noStrings, stringArrayClass, NULL);

	for (unsigned int i = 0; i < noStrings; i++){
		jstring jstring = env->NewStringUTF(c_strs[i]);
		env->SetObjectArrayElement(jStrings, i, jstring);
		env->DeleteLocalRef(jstring);
	//	DINFO(("Made java string (value = %s)\n", c_strs[i]));
	}
  
	DTRACE(("Out makeJavaStringArray\n"));
    return jStrings;
} 

JNIEXPORT jobjectArray JNICALL Java_cern_dip_implementation_DipBrowserImp__1getPublications
(JNIEnv * env, jobject, jint objectIndex, jstring filter){
	DTRACE(("In DipBrowserImp__1getPublications\n"));
	const char * filter_cstr = filter ? env->GetStringUTFChars(filter, NULL) : NULL;
	unsigned int noPubs;
	const char ** pubNames = browsePool.retrieve(objectIndex).getPublications(filter_cstr, noPubs);
	if (filter_cstr){
		env->ReleaseStringUTFChars(filter, filter_cstr);
	}
//	DINFO(("Building publication name list for %d pubs", noPubs));
	jobjectArray pubs = makeJavaStringArray(env, pubNames, noPubs);
	DTRACE(("Out DipBrowserImp__1getPublications\n"));
	return pubs;
}

JNIEXPORT jobjectArray JNICALL Java_cern_dip_implementation_DipBrowserImp__1getTags
  (JNIEnv *env, jobject, jint objectIndex, jstring pubName){
	DTRACE(("In DipBrowserImp__1getTags\n"));
	const char * pubName_cstr = pubName ? env->GetStringUTFChars(pubName, NULL) : NULL;
	unsigned int noFields;
	jobjectArray fieldList = NULL;

	try{
		const char ** fieldNames = browsePool.retrieve(objectIndex).getTags(pubName_cstr, noFields);
		DINFO(("Building tag list for %s with %d tags",pubName_cstr, noFields));
		fieldList = makeJavaStringArray(env, fieldNames, noFields);
	} catch (DipException &de){
		DipJavaCPPBridge::flagJavaDIPException(env, de.what());
	}
	if (pubName){
		env->ReleaseStringUTFChars(pubName, pubName_cstr);
	}
	DTRACE(("Out DipBrowserImp__1getTags\n"));
	return fieldList;
}

JNIEXPORT jint JNICALL Java_cern_dip_implementation_DipBrowserImp__1getType
  (JNIEnv * env, jobject, jint objectIndex, jstring field){
	DTRACE(("In DipBrowserImp__1getType\n"));
	const char * field_cstr = field ? env->GetStringUTFChars(field, NULL) : NULL;
	jint type = 0;
	try{
		type = browsePool.retrieve(objectIndex).getType(field_cstr);
	} catch (DipException &de){
		DipJavaCPPBridge::flagJavaDIPException(env, de.what());
	}
	if (field_cstr){
		env->ReleaseStringUTFChars(field, field_cstr);	
	}
	DTRACE(("Out DipBrowserImp__1getType\n"));
	return type;
}

JNIEXPORT jint JNICALL Java_cern_dip_implementation_DipBrowserImp__1getSize
  (JNIEnv *env, jobject, jint objectIndex, jstring field){
	DTRACE(("In DipBrowserImp__1getSize\n"));
	const char * field_cstr = field ? env->GetStringUTFChars(field, NULL) : NULL;
	jint cardinality = -1;
	try{
		cardinality = browsePool.retrieve(objectIndex).getSize(field_cstr);
	} catch (DipException &de){
		DipJavaCPPBridge::flagJavaDIPException(env, de.what());
	}

	if (field_cstr){
		env->ReleaseStringUTFChars(field, field_cstr);
	}
	DTRACE(("Out DipBrowserImp__1getSize\n"));
	return cardinality;
}
