/*
 * Created on Aug 17, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package cern.dip;
import cern.dip.implementation.DipFactoryImp;

/**
 * Bootstrapping object to start the DIPimplementation.
 * Instanciates the DIP factory object (singleton) as ensures only 1 instance is
 * ever created.
 */
public class Dip 
{
    /**
     * Call to the C++implementation to set up the underlying implementation.
     * */
	static private native void _setUpDIPServer(String serverName);
	
    /**
     * Call to the C++implementation to set up the underlying implementation.
     * */
	static private native String _getDIPVersion();
	
	/**
	 * name the server will have in the implementation name space 
	 * */
	static String dimServerName = null;


    /**
     *Reference to the DipFactory - this is a singleton.
     * */
	static DipFactory factory = null;
	
	
	/**
	 * Set up a DIPfactory for use as a client only. If the DipFactory has already
     * been created as a server, this call will have no effect.
     * @return the DipFactory instance configured for use as client
     * @see DipFactory
	 * */
	static public DipFactory create()
	{
		if (factory == null){
			_setUpDIPServer(null);
			factory = new DipFactoryImp();
		}
		
		return factory;
	}
	
	/**
	 * Get the current DIP version as a String.
     * @return The current DIP version as a String.
	 *
	 */
	static public String getDipVersion(){
	  return _getDIPVersion();
	}
	
	
	/**
	* Set up a DipFactory for up for use as client and/or server.
    *  If the DipFactory has already been created as a client, this call will
    * have no effect.
    * @param name Name of this DIM Server as declared to the name server (must be unique)
    * @return the DipFactory instance configured for use as client/server
    * @see DipFactory
	* */
	static public DipFactory create(String name)
	{
		if (factory == null){
			_setUpDIPServer(name);
			factory = new DipFactoryImp();
		}
		
		return factory;
	}
 
	/**
	* Set up a DipFactory for up for use as client and/or server.
    *  If the DipFactory has already been created as a client, this call will
    * have no effect.
    * @param name Name of this DIM Server as declared to the name server (must be unique)
    * @param dipnsServers Comma-separated lists of DIP name server hostnames (<i>e.g.</i> "dipns1,dipns2")
    * @since 5.5.5
    * @return the DipFactory instance configured for use as client/server
    * @see DipFactory
    * @see Dip#create()
	* */
	static public DipFactory create(String name, String dipnsServers)
	{
		if (factory == null){
			_setUpDIPServer(name);
			factory = new DipFactoryImp();
			factory.setDNSNode(dipnsServers);
		}
		
		return factory;
	}

 	/**
     *Anonymous static constructor - load the CPP implementation.
     * */
	static {
		// Load the DIP DLL when this class is loaded.
		System.loadLibrary("jDIP");
	}
}