package cern.dip;

/**
 * Interface allowing a user to browse the DIP namespace.
 */
public interface DipBrowser {    
   /**    
    * Obtain a list of all publications visable in the DIP namespace.
    * If no publications are visable an empty array is returned.
    * @param pattern wildcard string.     * @return array of publication names.
    * 
    */	
    public String[] getPublications(String pattern);
    
    /**
     * Get the fields names within the publication having the supplied publication name.
     * Should always be at least one field.
     * A primitive type will return the tag name __DIP_DEFAULT__
     * @param pub name of the publication we want the fields of.
     * @return Array of fields within the publication.
     * @throws DipException if the format discription of the publication is incorrect
     * 
     */	
    public String[] getTags(String pub) throws DipException;	
    
    /**     
     * Get the type of the default field (where a primitive type is being sent).
     * Must be called after first calling getTags() for the publication of interest.
     * @return type of the primitive sent in the DipData object as defined in the DipData class
     * @throws DipException if getTags() has not been previous called or the object does not hold a primitive type
     * @see DipBrowser.getTags().
     * @see DipData
     * 
     */
     public int getType()  throws DipException;	
     
     /**
      * Get the cardinality of the default field (where a primitive type is being sent).
      * Must be called after first calling getTags() for the publication of interest.
      * Note that a non array type will return the value 1.
      * @return cardinality of the primitive sent in the DipData object.
      * @throws DipException if getTags() has not been previous called or the object does not hold a primitive type
      * @see DipBrowser.getTags().
      * @see DipData
      * 
      */
     public int getSize() throws DipException;
     
     /**
      * Get the type of the field who's name is supplied as a parameter.
      * Must be called after first calling getTags() for the publication of interest.
      * @param name of the field whos type we are interested in.
      * @return type of the field sent in the DipData object as defined in the DipData class
      * @throws DipException if getTags() has not been previous called or the object does not hold a complex type
      * @see getTags()
      * @see DipData
      * 
      */
     public int getType(String tag) throws DipException;
     
     /**
      * Get the cardinality of the field who's name is supplied as a parameter.
      * Must be called after first calling getTags() for the publication of interest.
      * Note that a non array type will return the value 1.
      * @param name of the field whos cardinality we are interested in.
      * @return cardinality of the field sent in the DipData object as defined in the DipData class
      * @throws DipException if getTags() has not been previous called or the object does not hold a complex type
      * @see DipBrowser.getTags().
      * @see DipData
      * 
      */
     public int getSize(String tag) throws DipException;
}

