package cern.dip;

import java.io.Serializable;

/**
 * Should be used to indicate the time at which the value of a DipData Item was
 * obtained.
 */
public class DipTimestamp implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2325296406482833040L;
	private long timeNanoseconds = 0; // UTC nanoseconds

	/**
	 * Creates timestamp object with the current time.
	 */
	public DipTimestamp() {
		long timestamp = System.currentTimeMillis();
		timeNanoseconds = timestamp * 1000000;
	}

	/**
	 * Creates DipTimestamp from UTC Java time (in milliseconds).
	 */
	public DipTimestamp(long milliseconds) {
		timeNanoseconds = milliseconds * 1000000;
	}

	/**
	 * Sets UTC milliseconds.
	 */
	public void setMillis(long milliseconds) {
		timeNanoseconds = milliseconds * 1000000;
	}

	/**
	 * Sets UTC nanoseconds.
	 */
	public void setNanos(long nanoseconds) {
		timeNanoseconds = nanoseconds;
	}

	/**
	 * Get UTC millisec
	 */
	public long getAsMillis() {
		return timeNanoseconds / 1000000;
	}

	/**
	 * Get UTC nanoSec
	 */
	public long getAsNanos() {
		return timeNanoseconds;
	}
}