/*
 * Created on Aug 13, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package cern.dip.implementation;

import cern.dip.*;

/**
 * @author beharrel
 *
 */
public class DipSubscriptionImp extends FactoryProduct implements DipSubscription{
	
	/**
	 * Id returned by the C++ implementation
	 * */
	private int subscriptionID = -2;
	
	/**
	 * Handler for the subscription
	 * */
	private DipSubscriptionListener listener;
	
	/**
	 * Publication name;
	 * */
	private String name;
	
	
	/**
	 * Used because we need to create DipData
	 * */
	private DipFactory factory;
	
	
	private native int _create(String topicName) throws DipException;
	private native void _destroy(int objectID);
	
	
	private native void _requestUpdate(int objectID);
	
	
	
	public DipSubscriptionImp(DipFactory fact, String topicName, cern.dip.DipSubscriptionListener list)  throws DipException{
		name = topicName;
		factory = fact;
		listener = list;
		subscriptionID = _create(topicName);
	}
	
	
	public void finalize(){
		if (!isDestroyed() && (subscriptionID >= 0)){
			_destroy(subscriptionID);
		}
	}
	
	public String getTopicName() {
		return name;
	}

	public void requestUpdate() {
		if (this.isDestroyed()){
			return;
		}
		_requestUpdate(subscriptionID);
	}

	/**
	 * Delegates to the Listener. This is an entry point for the C++ implementation.
	 * */
	public void connected() {
		if (this.isDestroyed()){
			return;
		}
		try{
			listener.connected(this);
		} catch (Exception e){
			handleException(e);	
		}
	}

	/**
	 * Delegates to the Listener. This is an entry point for the C++ implementation.
	 * */
	public void disconnected(String reason) {
		if (this.isDestroyed()){
			return;
		}
		try{
			listener.disconnected(this, reason);
		} catch (Exception e){
			handleException(e);	
		}
	}

	/**
	 * Delegates to the Listener. This is an entry point for the C++ implementation.
	 * */
	public void handleException(Exception ex) {
		if (this.isDestroyed()){
			return;
		}
		try{
			listener.handleException(this, ex);
		} catch (Throwable e){
			System.out.println("PANIC!!: Subscription "+getTopicName()+" handleException() has thrown exception!");
			e.printStackTrace();
		}
	}

	/**
	 * Delegates to the Listener. This is an entry point for the C++ implementation.
	 * */
	public void handleMessage(String format, java.nio.ByteBuffer buffer, int quality, String qualityreason, long timeStamp_ns) {
		if (this.isDestroyed()){
			return;
		}
		DipDataImp data = (DipDataImp)factory.createDipData();
		try{
			data.readInFromBuffer(buffer, format);
		} catch (TypeMismatch e){
			handleException(new DipException("Error occured whilst deserialising data for " + getTopicName()));
			return;
		}
		data.setDataQuality(quality);
		DipTimestamp time = new DipTimestamp();
		time.setNanos(timeStamp_ns);
		data.setDipTime(time);
		data.setQualityString(qualityreason);
		try{
			listener.handleMessage(this, data);
		} catch (Exception e){
			handleException(e);	
		}
	}
	
	/**
	* Mark object as destoyed - will cause the C++ representation to be removed 
	*/
	protected void setDestroyed() {
		if (!destroyed){
			_destroy(subscriptionID);
        	destroyed = true;
        }
	}
}
