package cern.dip.mock;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import cern.dip.DipBrowser;
import cern.dip.DipData;
import cern.dip.DipException;
import cern.dip.DipFactory;
import cern.dip.DipPublication;
import cern.dip.DipPublicationErrorHandler;
import cern.dip.DipSubscription;
import cern.dip.DipSubscriptionListener;

public class MockDipFactory implements DipFactory {

	public HashMap<String, Collection<MockDipSubscription>> m_publicationNameToSubscription = new HashMap<String, Collection<MockDipSubscription>>();
	public HashMap<MockDipSubscription, DipSubscriptionListener> m_subscriptionToListener = new HashMap<MockDipSubscription, DipSubscriptionListener>();
	
	@Override
	public void setDNSNode(String nodeName) {
	}

	@Override
	public void setTimeout(int timeout) {
	}

	@Override
	public int getTimeout() {
		return 0;
	}

	@Override
	public DipSubscription createDipSubscription(String topicName,
			DipSubscriptionListener listener) throws DipException {
		MockDipSubscription result = new MockDipSubscription(this, topicName);
		m_subscriptionToListener.put(result, listener);
		
		if(!m_publicationNameToSubscription.containsKey(topicName)){
			m_publicationNameToSubscription.put(topicName, new ArrayList<MockDipSubscription>());
		}
		m_publicationNameToSubscription.get(topicName).add(result);
		
		
		return result;
	}

	@Override
	public DipPublication createDipPublication(String topicName,
			DipPublicationErrorHandler handler) throws DipException {
		MockDipPublication result = new MockDipPublication(this, topicName);
		result.addPublicationErrorHandler(handler);
		return result;
	}

	@Override
	public void destroyDipSubscription(DipSubscription subscription)
			throws DipException {
		
		m_publicationNameToSubscription.get(subscription.getTopicName()).remove(subscription);
		m_subscriptionToListener.remove(subscription);
	}

	@Override
	public void destroyDipPublication(DipPublication publication)
			throws DipException {
		// NOOP, the publication will be garbage collected
		// Note that we may still have subscribers, which should be updated if the publication
		// is recreated later.
	}

	@Override
	public DipData createDipData() {
		MockDipData result = new MockDipData();
		return result;
	}

	@Override
	public DipBrowser createDipBrowser() {
		MockDipBrowser browser = new MockDipBrowser();
		browser.dipFactory=this;
		return browser;
	}

	/**
	 * Use this method to programmatically cause on update on all registered subscription listeners.
	 * @param publication The publication that was updated.
	 * @param value  The DipData packet to send to all listeners.
	 */
	public void onPublicationUpdate(DipPublication publication,DipData value) {
		Collection<MockDipSubscription> subs = m_publicationNameToSubscription.get(publication.getTopicName());
		if(subs != null){
			for (MockDipSubscription mockDipSubscription : subs) {
				mockDipSubscription.setCurrentValue(value);
				m_subscriptionToListener.get(mockDipSubscription).handleMessage(mockDipSubscription, value);
			}
		}
	}

}
