package cern.dip.mock;

import cern.dip.DipData;
import cern.dip.DipException;
import cern.dip.DipFactory;
import cern.dip.DipPublication;
import cern.dip.DipPublicationErrorHandler;
import cern.dip.DipQualityEnum;
import cern.dip.DipTimestamp;
import cern.dip.implementation.DipDataImp;
import cern.dip.implementation.DipPublicationImpBase;

public class MockDipPublication extends DipPublicationImpBase {

	String m_topicName;
	private DipPublicationErrorHandler m_handler;
	private MockDipFactory m_factory;
	
	DipData m_lastKnownValue;
	DipQualityEnum m_quality = DipQualityEnum.GOOD;
	String m_qualityReason = "";
	DipTimestamp m_lastTimestamp;

	public MockDipPublication(MockDipFactory factory, String topicName) {
		m_topicName = topicName;
		m_factory = factory;
	}

	@Override
	public String getTopicName() {
		return m_topicName;
	}

	@Override
	protected DipFactory getFactory() {
		return m_factory;
	}

	public void addPublicationErrorHandler(DipPublicationErrorHandler handler) {
		m_handler = handler;
	}

	@Override
	public void send(DipData value, DipTimestamp timestamp) throws DipException {
		sendData(value, timestamp);
	}

	@Override
	public void send(DipData value, DipTimestamp timestamp, int dipQuality,
			String qualityReason) throws DipException {
		m_qualityReason = qualityReason;
		m_quality = DipQualityEnum.valueOf(dipQuality);
		sendData(value, timestamp);
	}

	@Override
	public void setQualityBad() throws DipException {
		m_quality = DipQualityEnum.BAD;
	}

	@Override
	public void setQualityBad(String reason) throws DipException {
		setQualityBad();
		m_qualityReason = reason;
		
	}

	@Override
	public void setQualityUncertain() throws DipException {
		m_quality = DipQualityEnum.UNCERTAIN;
	}

	@Override
	public void setQualityUncertain(String reason) throws DipException {
		setQualityUncertain();
		m_qualityReason = reason;
	}

	@Override
	protected void sendData(DipData value, DipTimestamp timestamp)
			throws DipException {
		m_lastTimestamp = timestamp;
		((MockDipData)value).setDipTime(timestamp);
		((MockDipData)value).setDataQuality(m_quality.intValue());
		((MockDipData)value).setQualityString(m_qualityReason);
		m_lastKnownValue = value;
		m_factory.onPublicationUpdate(this,value);
		
	}

	@Override
	protected void setDestroyed() {
	}

}
