package cern.dip.mock;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

import cern.dip.DipData;
import cern.dip.DipException;
import cern.dip.DipPublication;
import cern.dip.DipPublicationErrorHandler;
import cern.dip.DipSubscription;
import cern.dip.DipSubscriptionListener;
import cern.dip.DipSubscriptionListenerAdapter;
import cern.dip.TypeMismatch;
import junit.framework.TestCase;


public class DipMockFactoryTest extends TestCase {

	public void testSimpleFactory() throws DipException{
		MockDipFactory factory = new MockDipFactory();
		DipPublication pub = factory.createDipPublication("dip/mock/testPub", new DipPublicationErrorHandler() {
			@Override
			public void handleException(DipPublication publication, DipException ex) {
				fail("Failed with exception "+ex);
				ex.printStackTrace();
			}
		});
		final AtomicBoolean receivedMessage = new AtomicBoolean(false);
		DipSubscription sub = factory.createDipSubscription("dip/mock/testPub", new DipSubscriptionListenerAdapter() {
			@Override
			public void handleMessage(DipSubscription subscription, DipData message) {
				try {
					if(subscription.getTopicName().equals("dip/mock/testPub") && message.extractBoolean()){
					  receivedMessage.set(true);
					}
				} catch (TypeMismatch e) {
					fail("Failed with exception "+e.getMessage());
					e.printStackTrace();
				}
			}
			
			@Override
			public void handleException(DipSubscription subscription, Exception ex) {
				fail("Failed with exception "+ex);
				ex.printStackTrace();
			}
			
		});
		DipData dipData = factory.createDipData();
		dipData.insert(true);
		factory.onPublicationUpdate(pub, dipData);
		
		assertTrue(receivedMessage.get());
	}
	
	public void testDestroySubscription() throws DipException{
		MockDipFactory factory = new MockDipFactory();
		DipPublication pub = factory.createDipPublication("dip/mock/testPub", new DipPublicationErrorHandler() {
			@Override
			public void handleException(DipPublication publication, DipException ex) {
				fail("Failed with exception "+ex);
				ex.printStackTrace();
			}
		});
		final AtomicInteger receivedCount = new AtomicInteger();
		DipSubscriptionListener listener =  new DipSubscriptionListenerAdapter() {
			@Override
			public void handleMessage(DipSubscription subscription, DipData message) {
				try {
					if(subscription.getTopicName().equals("dip/mock/testPub") && message.extractBoolean()){
						receivedCount.incrementAndGet();
					}
				} catch (TypeMismatch e) {
					fail("Failed with exception "+e.getMessage());
					e.printStackTrace();
				}
			}
			
			@Override
			public void handleException(DipSubscription subscription, Exception ex) {
				fail("Failed with exception "+ex);
				ex.printStackTrace();
			}
			
		};
		DipSubscription sub = factory.createDipSubscription("dip/mock/testPub",listener);
		DipData dipData = factory.createDipData();
		dipData.insert(true);
		factory.onPublicationUpdate(pub, dipData);
		
		assertEquals(1,receivedCount.get());
		
		factory.onPublicationUpdate(pub, dipData);
		assertEquals(2,receivedCount.get());
		
		factory.destroyDipSubscription(sub);
		factory.onPublicationUpdate(pub, dipData);
		assertEquals(2,receivedCount.get());
		
		DipSubscription sub2 = factory.createDipSubscription("dip/mock/testPub",listener);
		factory.onPublicationUpdate(pub, dipData);
		assertEquals(3,receivedCount.get());
		
	}
}
