# Introduction

The DIP Browser JNLP code must be digitally signed to be able to run on Java 7 or later, without forcing the user to register a security exception.

CERN provides code signing certificates for Service Accounts and ensure that all centrally managed Java distributions ships with a CERN CA Root Certificate.

With a CERN-issued code certificate, one can sign the Java JAR packages, and ensure they have not been tampered with after deployment.

This procedure explains how to proceed to renew the DIP Browser code signing certificate.

# Procedure

## Pre-requisites :

* You must have ownership of the DIPNS CERN service account (i.e. know the account password), this is the service account used to run the DIP name servers and ensure.

## Using CERN CA autogenerated code signing certificates

* Login to http://cern.ch/ca with a service account and request a code signing certificate.
* IMPORTANT : Set a password on the certificate (or you won't be able to import it with java keytool)
* delete the existing certificate if any

    keytool -delete -keystore dip.keystore -alias ??

* execute the following to import the certificate into the keystore

    keytool -importkeystore -deststorepass dipmet -destkeystore dip.keystore -srckeystore dipns-code-signing-certificate.p12 -srcstoretype PKCS12

* rename the alias to DIPNS (by default, the alias is "1")

    keytool -changealias -keystore dip.keystore -alias 1 -destalias dipns

## Using a CSR (old procedure)
* Request a code signing certificate from http://cern.ch/gridca using the DIPNS account.
   * You can execute the openssl command from LXPLUS for instance
   * Generate a private key if necessary, and save it as "privkey.pem" (default name).
   * The PEM password is typically the service account password (you will need this password later to reimport the certificate and private key into the keystore)
   * (!) SAVE the BASE64 encoded certificate file as *dipns.b64.cer*, as this is the one we will use to import into the DIP java keystore

* We must now merge the PEM private key and the certificate into a different format (P12 format), to import both into the Java keystore. Save the PEM and the certificate, along with the CERN CA ROOT certificate if not already available locally.

* Use the following keytool command on LXPLUS to generate the .P12 format certificate

    openssl pkcs12 -export -in dipns.b64.cer -inkey privkey.pem -out dipns.p12 -name dipns -CAfile CERN\ Root\ Certification\ Authority\ 2.crt -caname CERN -chain

* This should generate a *dipns.p12* file, including the full verification chain.

* Delete the previous certificate entry with

    keytool -delete -keystore dip.keystore -alias dip

* Import the P12 certificate into your keystore, with the following command

    keytool -importkeystore -deststorepass dipmet -destkeystore dip.keystore -srckeystore dipns.p12 -srcstoretype PKCS12 -srcstorepass <DIPNS account password>


