package cern.dip.mqtt.listener;

import java.util.concurrent.TimeUnit;

import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.influxdb.dto.Point;
import org.influxdb.dto.Point.Builder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import cern.dip.DipData;
import cern.dip.DipSubscription;
import cern.dip.mqtt.Subscriber;
import cern.dip.mqtt.SubscriberConfig;
import lombok.extern.slf4j.Slf4j;

@Component
@ConditionalOnProperty(prefix="dip.mqtt.subscriber.outputs",name="influxdb", havingValue="true")
@Slf4j
public class InfluxDbFormatListener implements DipUpdateFormatter{
    
    @Autowired
    SubscriberConfig config;
    
    @Autowired
    Subscriber parentSubscriber;
    
    public InfluxDbFormatListener(SubscriberConfig config, Subscriber parentSubscriber) {
        this.config = config;
        this.parentSubscriber = parentSubscriber;
        this.parentSubscriber.addDipUpdateListener(this);
    }
    
    @Override
    public MqttMessage prepareMqttMessage(DipSubscription subscription,DipData dipData) {
        try {
            Builder pointBuilder = Point.measurement(subscription.getTopicName())
                    .time(dipData.extractDipTime().getAsMillis(), TimeUnit.MILLISECONDS);
            
            pointBuilder.tag("qflg", Integer.toString(dipData.extractDataQuality()));
            pointBuilder.tag("qstr", dipData.extractQualityString());
            
            pointBuilder.fields(parentSubscriber.convertDipDataToObject(dipData));
            
            Point point = pointBuilder.build();
            MqttMessage message = new MqttMessage(point.lineProtocol().getBytes());
            
            message.setQos(0);
            message.setRetained(true);

            return message;
        } catch (Exception e) {
            log.error("Exception while pushing message : {}", e);
        }
        return null;
    }
    
    

   
}
