package cern.dip.mqtt.listener;

import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.databind.ObjectMapper;

import cern.dip.DipData;
import cern.dip.DipSubscription;
import cern.dip.mqtt.DipJsonDTO;
import cern.dip.mqtt.Subscriber;
import cern.dip.mqtt.SubscriberConfig;
import lombok.extern.slf4j.Slf4j;

@Component
@ConditionalOnProperty(prefix="dip.mqtt.subscriber.outputs",name="json", havingValue="true")
@Slf4j
public class JsonFormatListener implements DipUpdateFormatter{
    
    @Autowired
    SubscriberConfig config;
    
    @Autowired
    Subscriber parentSubscriber;
    
    ObjectMapper objectMapper = new ObjectMapper();
    
    public JsonFormatListener(SubscriberConfig config, Subscriber parentSubscriber) {
        this.config = config;
        this.parentSubscriber = parentSubscriber;
        this.parentSubscriber.addDipUpdateListener(this);
        objectMapper.setSerializationInclusion(Include.NON_NULL);
    }
    
    @Override
    public MqttMessage prepareMqttMessage(DipSubscription subscription,DipData dipData) {
        try {
            DipJsonDTO dto = new DipJsonDTO();
            if(config.getSerializePublicationName()){
                dto.setPub(subscription.getTopicName());
            }
            dto.setValue(parentSubscriber.convertDipDataToObject(dipData));
            dto.setQflg(dipData.extractDataQuality());
            dto.setQstr(dipData.extractQualityString());
            dto.setTs( dipData.extractDipTime().getAsMillis());
            MqttMessage message = new MqttMessage(objectMapper.writeValueAsBytes(dto));
            message.setQos(0);
            message.setRetained(true);
            return message;
        } catch (Exception e) {
            log.error("Exception while pushing message : {}", e);
        }
        return null;
    }
   
}
