/*
 * DIP Web Browser
 *
 * Copyright (C) CERN 2015 All rights reserved.
 */
/*
 * you have to provide the following variables:
 * 
 * var restServiceBaseUrl = "http://broadcaster.example/broadcast-servlet";
 * var requestServiceUrl = restServiceBaseUrl + '/broadcast/';
 * var subscribeServiceUrl = restServiceBaseUrl + "/subscribe/";
 * var subscriptionJSONListUrl = "js/subscriptions.js";
 * var subscriptionTopic = "subscriptionTopic";
 * 
*/
QUnit.test("Subscribe test", function(assert) {

	function dummyDataParsing(data) {
		console.log("dummyDataParsing(data)");
		console.log(data);
		assert.notEqual(data.topicName, null, "Topic name not null in data");
		assert.equal(data.topicName, subscriptionTopic, "Topic name matches");
	}

	var connection = new BroadcastConnection(restServiceBaseUrl,
		subscriptionTopic, dummyDataParsing);

	assert.equal(connection.requestServiceUrl, requestServiceUrl, "Checking requestServiceUrl...");
	assert.equal(connection.subscribeServiceUrl, subscribeServiceUrl, "Checking subscribeServiceUrl...");
	assert.equal(connection.subscriptionTopic, subscriptionTopic, "Checking subscriptionTopic...");

	var done1 = assert.async();
	var done2 = assert.async();
	var subs = [
		"anx://el/cryo-2015.01.01/signal/query?startDate=1420095000000&endDate=1420100000000&interval=180&agg=avg",
		"anx://el/~cryo-.*/signal/query?startDate=1420095000000&endDate=1420100000000&prefix=arc23" ];

	connection.getSubscriptionsList(subscriptionJSONListUrl, function() {
		assert.deepEqual(connection.subscriptionsList, subs, "Loading subscriptions list...");
		done1();
		connection.topicSubscribe(null, function() {
			var connected = connection.connected();
			assert.ok(connected, "Socket connected, subscription ok");
			done2();
		});
	});

});
